# TestSwiftConsumeOperatorAsync.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Check that we properly show variables at various points of the CFG while
stepping with the consume operator.
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import sys
import unittest2

def stderr_print(line):
    sys.stderr.write(line + "\n")

class TestSwiftConsumeOperatorAsyncType(TestBase):
    @swiftTest
    @skipIf(bugnumber="rdar://133849022", oslist=['linux'])
    def test_swift_consume_operator_async(self):
        """Check that we properly show variables at various points of the CFG while
        stepping with the consume operator.
        """
        self.build()

        self.breakpoints = []
        self.target, self.process, self.thread, bkpt = \
            lldbutil.run_to_source_breakpoint(
                self, 'Set breakpoint 00', lldb.SBFileSpec('main.swift'))
        self.breakpoints.append(bkpt)

        # We setup a single breakpoint in copyable var test so we can disable it
        # after we hit it.
        self.do_setup_breakpoints()

        self.do_check_copyable_value_test()
        self.do_check_copyable_var_test()

    def get_var(self, name):
        frame = self.thread.frames[0]
        return frame.FindVariable(name)

    def do_setup_breakpoints(self):
        for i in range(1, 11):
            bkpt = self.target.BreakpointCreateBySourceRegex(
                'Set breakpoint %02d'%i, lldb.SBFileSpec('main.swift'))
            self.assertGreater(bkpt.GetNumLocations(), 0, VALID_BREAKPOINT)
            self.breakpoints.append(bkpt)
        self.assertEqual(len(self.breakpoints), 11)

    def continue_to(self, bkpt_id):
        while self.process.is_alive and \
              not lldbutil.continue_to_breakpoint(self.process,
                                                  self.breakpoints[bkpt_id]):
            pass
        self.assertTrue(self.process.is_alive)

    def do_check_copyable_value_test(self):
        # We haven't defined varK yet.
        varK = self.get_var('k')
        self.assertEqual(varK.unsigned, 0, "varK initialized too early?!")

        # Go to break point 1.1. k should be valid.
        self.continue_to(1)
        varK = self.get_var('k')
        self.assertGreater(varK.unsigned, 0, "varK not initialized?!")

        # Go to breakpoint `1.2. k should still be valid. And we should be on the
        # other side of the force split.
        self.continue_to(1)
        varK = self.get_var('k')
        self.assertGreater(varK.unsigned, 0, "varK not initialized?!")

        # Go to breakpoint 2. k should still be valid. We should be at the move
        # on the other side of the forceSplit.
        self.continue_to(2)
        varK = self.get_var('k')
        self.assertGreater(varK.unsigned, 0, "varK not initialized?!")

        # We are now at break point 3. We have consumed k, it should be empty.
        self.continue_to(3)
        varK = self.get_var('k')
        self.assertIsNone(varK.value, "K is live but was consumed?!")

        # Finally, we are on the other side of the final force split. Make sure
        # the value still isn't available.
        self.continue_to(4)
        varK = self.get_var('k')
        self.assertIsNone(varK.value, "K is live but was consumed?!")

    def do_check_copyable_var_test(self):
        # Run so we hit the next breakpoint to jump to the next test's
        # breakpoint.
        self.continue_to(5)
        # We haven't defined varK yet.
        varK = self.get_var('k')
        self.assertEqual(varK.unsigned, 0, "varK initialized too early?!")

        # Go to break point 6.1. k should be valid.
        self.continue_to(6)
        varK = self.get_var('k')
        self.assertGreater(varK.unsigned, 0, "varK not initialized?!")

        # Go to breakpoint 6.2. k should still be valid. And we should be on the
        # other side of the force split.
        self.continue_to(6)
        varK = self.get_var('k')
        self.assertGreater(varK.unsigned, 0, "varK not initialized?!")

        # Go to breakpoint 7. k should still be valid. We should be at the move
        # on the other side of the forceSplit.
        self.continue_to(7)
        varK = self.get_var('k')
        self.assertGreater(varK.unsigned, 0, "varK not initialized?!")

        # We are now at break point 8. We have consumed k, it should be empty.
        self.continue_to(8)
        varK = self.get_var('k')
        self.assertIsNone(varK.value, "K is live but was consumed?!")

        # Now, we are on the other side of the final force split. Make sure
        # the value still isn't available.
        self.continue_to(9)
        self.runCmd('# On other side of force split')
        varK = self.get_var('k')
        self.assertIsNone(varK.value, "K is live but was consumed?!")

        # Finally, we have reinitialized k, look for k.
        self.continue_to(10)
        self.runCmd('# After var reinit')
        varK = self.get_var('k')
        self.assertGreater(varK.unsigned, 0, "varK not initialized?!")

        self.runCmd('# At end of routine!')



# TestPlaygrounds.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that playgrounds work
"""
import subprocess
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import os.path
import platform
import unittest2
from lldbsuite.test.builders.darwin import get_triple

import sys
if sys.version_info.major == 2:
    import commands as subprocess
else:
    import subprocess


def execute_command(command):
    (exit_status, output) = subprocess.getstatusoutput(command)
    return exit_status


class TestSwiftPlaygrounds(TestBase):
    def get_build_triple(self):
        """We want to build the file with a deployment target earlier than the
           availability set in the source file."""
        if lldb.remote_platform:
            arch = self.getArchitecture()
            vendor, os, version, _ = get_triple()
            # This is made slightly more complex by watchOS having misaligned
            # version numbers.
            if os == 'watchos':
                version = '5.0'
            else:
                version = '7.0'
            triple = '{}-{}-{}{}'.format(arch, vendor, os, version)
        else:
            triple = '{}-apple-macosx11.0'.format(platform.machine())
        return triple

    def get_run_triple(self):
        if lldb.remote_platform:
            arch = self.getArchitecture()
            vendor, os, version, _ = get_triple()
            triple = '{}-{}-{}{}'.format(arch, vendor, os, version)
        else:
            version, _, machine = platform.mac_ver()
            triple = '{}-apple-macosx{}'.format(machine, version)
        return triple

    @skipUnlessDarwin
    @swiftTest
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(debug_info=decorators.no_match("dsym"))
    def test_force_target(self):
        """Test that playgrounds work"""
        self.launch(True)
        self.do_basic_test(True)

    @skipUnlessDarwin
    @swiftTest
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(debug_info=decorators.no_match("dsym"))
    def test_no_force_target(self):
        """Test that playgrounds work"""
        self.launch(False)
        self.do_basic_test(False)

    @skipUnlessDarwin
    @swiftTest
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(debug_info=decorators.no_match("dsym"))
    @skipIf(macos_version=["<", "12"])
    def test_concurrency(self):
        """Test that concurrency is available in playgrounds"""
        self.launch(True)
        self.do_concurrency_test()

    @skipUnlessDarwin
    @swiftTest
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(debug_info=decorators.no_match("dsym"))
    def test_import(self):
        """Test that a dylib can be imported in playgrounds"""
        self.launch(True)
        self.do_import_test()
        
    def launch(self, force_target):
        """Test that playgrounds work"""
        self.build(dictionary={
            'TARGET_SWIFTFLAGS':
            '-target {}'.format(self.get_build_triple()),
        })

        # Create the target
        exe = self.getBuildArtifact("PlaygroundStub")
        if force_target:
            target = self.dbg.CreateTargetWithFileAndArch(
                exe, self.get_run_triple())
        else:
            target = self.dbg.CreateTarget(exe)

        self.assertTrue(target, VALID_TARGET)
        self.registerSharedLibrariesWithTarget(target,
                                               ['libPlaygroundsRuntime.dylib'])

        # Set the breakpoints
        breakpoint = target.BreakpointCreateBySourceRegex(
            'Set breakpoint here', lldb.SBFileSpec("PlaygroundStub.swift"))
        self.assertTrue(breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT)

        process = target.LaunchSimple(None, None, os.getcwd())
        self.assertTrue(process, PROCESS_IS_VALID)

        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, breakpoint)

        self.assertEqual(len(threads), 1)
        self.expect('settings set target.swift-framework-search-paths "%s"' %
                    self.getBuildDir())

    def execute_code(self, input_file, expect_error=False):
        contents = "syntax error"
        with open(input_file, 'r') as contents_file:
            contents = contents_file.read()

        options = lldb.SBExpressionOptions()
        options.SetLanguage(lldb.eLanguageTypeSwift)
        options.SetPlaygroundTransformEnabled()
        # The concurrency expressions will spawn multiple threads.
        options.SetOneThreadTimeoutInMicroSeconds(1)
        options.SetTryAllThreads(True)
        options.SetAutoApplyFixIts(False)

        res = self.frame().EvaluateExpression(contents, options)
        ret = self.frame().EvaluateExpression("get_output()")
        is_error = res.GetError().Fail() and not (
                     res.GetError().GetType() == 1 and
                     res.GetError().GetError() == 0x1001)
        playground_output = ret.GetSummary()
        with recording(self, self.TraceOn()) as sbuf:
            print("playground result: ", file=sbuf)
            print(str(res), file=sbuf)
            if is_error:
                print("error:", file=sbuf)
                print(str(res.GetError()), file=sbuf)
            else:
                print("playground output:", file=sbuf)
                print(str(ret), file=sbuf)

        if expect_error:
            self.assertTrue(is_error)
            return playground_output
        self.assertFalse(is_error)
        self.assertIsNotNone(playground_output)
        return playground_output
        
    def do_basic_test(self, force_target):
        playground_output = self.execute_code('Contents.swift', not force_target)
        if not force_target:
            # This is expected to fail because the deployment target
            # is less than the availability of the function being
            # called.
            self.assertEqual(playground_output, '""')
            return

        self.assertIn("a=\\'3\\'", playground_output)
        self.assertIn("b=\\'5\\'", playground_output)
        self.assertIn("=\\'8\\'", playground_output)
        self.assertIn("=\\'11\\'", playground_output)

    def do_concurrency_test(self):
        playground_output = self.execute_code('Concurrency.swift')
        self.assertIn("=\\'23\\'", playground_output)

    def do_import_test(self):
        # Test importing a library that adds new Clang options.
        log = self.getBuildArtifact('types.log')
        self.expect('log enable lldb types -f ' + log)
        playground_output = self.execute_code('Import.swift')
        self.assertIn("Hello from the Dylib", playground_output)

        # Scan through the types log to make sure the SwiftASTContext was poisoned.
        self.filecheck('platform shell cat ""%s"' % log, __file__)
#       CHECK: New Swift image added{{.*}}Versions/A/Dylib{{.*}}ClangImporter needs to be reinitialized



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test_invalid_arg(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        self.expect("target create %s" % (exe))
        self.expect(
            "target modules search-paths insert -1 a b",
            error=True,
            startstr="error: <index> parameter is not an integer: '-1'.",
        )

        self.expect(
            "target modules search-paths insert abcdefx a b",
            error=True,
            startstr="error: <index> parameter is not an integer: 'abcdefx'.",
        )



from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil

class TestSwiftLateDylib(TestBase):
    @skipUnlessDarwin
    @swiftTest
    @skipIfDarwinEmbedded 
    def test(self):
        """Test that a late loaded Swift dylib is debuggable"""
        self.build()
        target, process, _, _ = lldbutil.run_to_name_breakpoint(self, "main")
        # Initialize SwiftASTContext before loading the dylib.
        self.expect("expr -l Swift -- 0")
        bkpt = target.BreakpointCreateByLocation(lldb.SBFileSpec('dylib.swift'), 7)
        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect("v x", substrs=['Hello from the Dylib'])
        self.expect("expr x", substrs=['Hello from the Dylib'])



"""
Test that the Python operating system plugin works correctly
"""


import os
import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class PluginPythonOSPlugin(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_python_os_plugin(self):
        """Test that the Python operating system plugin works correctly"""
        self.build()
        self.run_python_os_funcionality()

    def run_python_os_step(self):
        """Test that the Python operating system plugin works correctly when single stepping a virtual thread"""
        self.build()
        self.run_python_os_step()

    def verify_os_thread_registers(self, thread):
        frame = thread.GetFrameAtIndex(0)
        registers = frame.GetRegisters().GetValueAtIndex(0)
        reg_value = thread.GetThreadID() + 1
        for reg in registers:
            self.assertEqual(
                reg.GetValueAsUnsigned(),
                reg_value,
                "Verify the registers contains the correct value",
            )
            reg_value = reg_value + 1

    def run_python_os_funcionality(self):
        """Test that the Python operating system plugin works correctly"""

        # Set debugger into synchronous mode
        self.dbg.SetAsync(False)

        # Create a target by the debugger.
        exe = self.getBuildArtifact("a.out")
        python_os_plugin_path = os.path.join(self.getSourceDir(), "operating_system.py")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set breakpoints inside and outside methods that take pointers to the
        # containing struct.
        lldbutil.run_break_set_by_source_regexp(self, "// Set breakpoint here")

        # Register our shared libraries for remote targets so they get
        # automatically uploaded
        arguments = None
        environment = None

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            arguments, environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # Make sure there are no OS plug-in created thread when we first stop
        # at our breakpoint in main
        thread = process.GetThreadByID(0x111111111)
        self.assertFalse(
            thread.IsValid(),
            "Make sure there is no thread 0x111111111 before we load the python OS plug-in",
        )
        thread = process.GetThreadByID(0x222222222)
        self.assertFalse(
            thread.IsValid(),
            "Make sure there is no thread 0x222222222 before we load the python OS plug-in",
        )
        thread = process.GetThreadByID(0x333333333)
        self.assertFalse(
            thread.IsValid(),
            "Make sure there is no thread 0x333333333 before we load the python OS plug-in",
        )

        # Now load the python OS plug-in which should update the thread list and we should have
        # OS plug-in created threads with the IDs: 0x111111111, 0x222222222,
        # 0x333333333
        command = (
            "settings set target.process.python-os-plugin-path '%s'"
            % python_os_plugin_path
        )
        self.dbg.HandleCommand(command)

        # Verify our OS plug-in threads showed up
        thread = process.GetThreadByID(0x111111111)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x111111111 after we load the python OS plug-in",
        )
        self.verify_os_thread_registers(thread)
        thread = process.GetThreadByID(0x222222222)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x222222222 after we load the python OS plug-in",
        )
        self.verify_os_thread_registers(thread)
        thread = process.GetThreadByID(0x333333333)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x333333333 after we load the python OS plug-in",
        )
        self.verify_os_thread_registers(thread)

        # Now clear the OS plug-in path to make the OS plug-in created threads
        # disappear
        self.dbg.HandleCommand("settings clear target.process.python-os-plugin-path")

        # Verify the threads are gone after unloading the python OS plug-in
        thread = process.GetThreadByID(0x111111111)
        self.assertFalse(
            thread.IsValid(),
            "Make sure there is no thread 0x111111111 after we unload the python OS plug-in",
        )
        thread = process.GetThreadByID(0x222222222)
        self.assertFalse(
            thread.IsValid(),
            "Make sure there is no thread 0x222222222 after we unload the python OS plug-in",
        )
        thread = process.GetThreadByID(0x333333333)
        self.assertFalse(
            thread.IsValid(),
            "Make sure there is no thread 0x333333333 after we unload the python OS plug-in",
        )

    def run_python_os_step(self):
        """Test that the Python operating system plugin works correctly and allows single stepping of a virtual thread that is backed by a real thread"""

        # Set debugger into synchronous mode
        self.dbg.SetAsync(False)

        # Create a target by the debugger.
        exe = self.getBuildArtifact("a.out")
        python_os_plugin_path = os.path.join(
            self.getSourceDir(), "operating_system2.py"
        )
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set breakpoints inside and outside methods that take pointers to the
        # containing struct.
        lldbutil.run_break_set_by_source_regexp(self, "// Set breakpoint here")

        # Register our shared libraries for remote targets so they get
        # automatically uploaded
        arguments = None
        environment = None

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            arguments, environment, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        # Make sure there are no OS plug-in created thread when we first stop
        # at our breakpoint in main
        thread = process.GetThreadByID(0x111111111)
        self.assertFalse(
            thread.IsValid(),
            "Make sure there is no thread 0x111111111 before we load the python OS plug-in",
        )

        # Now load the python OS plug-in which should update the thread list and we should have
        # OS plug-in created threads with the IDs: 0x111111111, 0x222222222,
        # 0x333333333
        command = (
            "settings set target.process.python-os-plugin-path '%s'"
            % python_os_plugin_path
        )
        self.dbg.HandleCommand(command)

        # Verify our OS plug-in threads showed up
        thread = process.GetThreadByID(0x111111111)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x111111111 after we load the python OS plug-in",
        )

        frame = thread.GetFrameAtIndex(0)
        self.assertTrue(
            frame.IsValid(), "Make sure we get a frame from thread 0x111111111"
        )
        line_entry = frame.GetLineEntry()

        self.assertEqual(
            line_entry.GetFileSpec().GetFilename(),
            "main.c",
            "Make sure we stopped on line 5 in main.c",
        )
        self.assertEqual(
            line_entry.GetLine(), 5, "Make sure we stopped on line 5 in main.c"
        )

        # Now single step thread 0x111111111 and make sure it does what we need
        # it to
        thread.StepOver()

        frame = thread.GetFrameAtIndex(0)
        self.assertTrue(
            frame.IsValid(), "Make sure we get a frame from thread 0x111111111"
        )
        line_entry = frame.GetLineEntry()

        self.assertEqual(
            line_entry.GetFileSpec().GetFilename(),
            "main.c",
            "Make sure we stepped from line 5 to line 6 in main.c",
        )
        self.assertEqual(
            line_entry.GetLine(),
            6,
            "Make sure we stepped from line 5 to line 6 in main.c",
        )



"""
Test completion for multiline expressions.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class MultilineCompletionTest(PExpectTest):
    def start_expression_editor(self):
        """Starts the multiline expression editor."""
        self.child.send("expression\n")
        self.child.expect_exact("terminate with an empty line to evaluate")

    def exit_expression_editor(self):
        """Exits the multiline expression editor."""
        # Send a newline to finish the current line. The second newline will
        # finish the new empty line which will exit the editor. The space at the
        # start prevents that the first newline already exits the editor (in
        # case the current line of the editor is already empty when this
        # function is called).
        self.child.send(" \n\n")
        self.expect_prompt()

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIfEditlineSupportMissing
    @expectedFailureAll(oslist=["freebsd"], bugnumber="llvm.org/pr49408")
    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])
    def test_basic_completion(self):
        """Test that we can complete a simple multiline expression"""
        self.build()

        self.launch(executable=self.getBuildArtifact("a.out"), dimensions=(100, 500))
        self.expect("b main", substrs=["Breakpoint 1", "address ="])
        self.expect("run", substrs=["stop reason = breakpoint 1"])

        self.start_expression_editor()
        self.child.send("to_\t")
        # editline might move the cursor back to the start of the line via \r
        # and then back to its original position.
        self.child.expect(
            re.compile(
                b"to_(\r"
                + self.cursor_forward_escape_seq(len("  1: to_"))
                + b")?complete"
            )
        )
        self.exit_expression_editor()

        # Check that completion empty input in a function with only one
        # local variable works.
        self.expect(
            "breakpoint set -p 'break in single_local_func'", substrs=["Breakpoint 2"]
        )
        self.expect("continue", substrs=["stop reason = breakpoint 2"])
        self.start_expression_editor()
        self.child.send("\t")
        # Only one local, so this will directly insert 'only_local' with a
        # trailing space to signal a final completion.
        self.child.expect_exact("only_local ")
        self.exit_expression_editor()

        self.quit()



"""
Test the swift metadata cache works correctly
"""
import glob
import io
import os
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftMetadataCache(TestBase):

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        cache_dir = self.getBuildArtifact('swift-metadata-cache')
        # Set the lldb module cache directory to a directory inside the build
        # artifacts directory so no other tests are interfered with.
        self.runCmd('settings set symbols.swift-metadata-cache-path "%s"' % cache_dir)
        self.runCmd('settings set symbols.enable-swift-metadata-cache true')

    def check_strings_in_log(self, log_path, strings):
        types_log = io.open(log_path, "r", encoding='utf-8')
        for line in types_log:
            # copy to remove from original
            copy = strings[:]
            for s in copy:
                if s in line:
                    try:
                        strings.remove(s)
                    # In case the string shows up more than once
                    except ValueError:
                        pass

            if len(strings) == 0:
                return True
        return False
            
        
    @swiftTest
    def test_swift_metadata_cache(self):
        """Test the swift metadata cache."""
        
        # This test runs does three runs:

        # In the first run, we make sure we build the cache.

        # In the second run, we make sure we use the cache.

        # In the third run, we emulate a source file modification by building 
        # main_modified.swift instead of main.swift. We then make sure that
        # we invalidate the modified cache and rebuild it.

        # Build with the "original" main file.
        self.build(dictionary={'SWIFT_SOURCES': 'main.swift'})

        types_log = self.getBuildDir() + "/first_run_types.txt"
        self.runCmd('log enable lldb types -v -f "%s"' % types_log)
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        # Run frame variable for the first time to build the cache.
        self.expect('v v', substrs=['SomeTypeWeWillLookUp'])

        # Check that we wrote the cache for the main module.
        self.assertTrue(self.check_strings_in_log(types_log, [
            'Cache file written for module a.out.']))
        # Finish run.
        self.runCmd('c')


        types_log = self.getBuildDir() + "/second_run_types.txt"
        self.runCmd('log enable lldb types -v -f "%s"' % types_log)
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        # Run frame variable for the second time to check if the cache was queried.
        self.expect('v v', substrs=['SomeTypeWeWillLookUp'])

        # Check that we have a cache and that we found the type of 'v' in it.
        self.assertTrue(self.check_strings_in_log(types_log, [
            'Loaded cache for module a.out.',
            'Returning field descriptor for mangled name 1a20SomeTypeWeWillLookUpV']))
        
        # Finish run.
        self.runCmd('c')

        
        # Rebuild the "modified" program. This should have the exact same path 
        # as the original one.
        self.build(dictionary={'SWIFT_SOURCES': 'main_modified.swift'})
        
        types_log = self.getBuildDir() + "/third_run_types.txt"
        self.runCmd('log enable lldb types -v -f "%s"' % types_log)
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main_modified.swift'))
         

        # Run frame variable for the third time to check that the cache is invalidated
        # and that we rebuild it.
        self.expect('v v', substrs=['SomeTypeWeWillLookUp'])

        # Check that we found the type of 'v' in the cache.
        self.assertTrue(self.check_strings_in_log(types_log, [
            'Module UUID mismatch for a.out.',
            'Cache file written for module a.out.']))



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil

class TestCase(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    @skipIf(oslist=['windows', 'linux'])
    def test(self):
        """Test `frame variable` in async functions"""
        self.build()

        source_file = lldb.SBFileSpec("main.swift")
        target, process, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break one", source_file)

        # At "break one", only the `a` variable should have a value.
        frame = process.GetSelectedThread().frames[0]
        a = frame.FindVariable("a")
        self.assertTrue(a.IsValid())
        self.assertGreater(a.unsigned, 0)
        b = frame.FindVariable("b")
        self.assertFalse(b.IsValid())
        d = frame.FindVariable("d")
        lldbutil.check_variable(self, d, False, value='23')

        # The first breakpoint resolves to multiple locations, but only the
        # first location is needed. Now that we've stopped, delete it to
        # prevent the other locations from interrupting the test.
        target.DeleteAllBreakpoints()

        # Setup, and run to, the next breakpoint.
        target.BreakpointCreateBySourceRegex("break two", source_file)
        self.setAsync(False)
        process.Continue()

        # At "break two", both `a` and `b` should have values.
        frame = process.GetSelectedThread().frames[0]
        a = frame.FindVariable("a")
        self.assertTrue(a.IsValid())
        self.assertGreater(a.unsigned, 0)
        b = frame.FindVariable("b")
        self.assertTrue(b.IsValid())
        self.assertGreater(b.unsigned, 0)
        d = frame.FindVariable("d")
        lldbutil.check_variable(self, d, False, value='23')



from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil

class TestSwiftTypeAlias(TestBase):
    @swiftTest
    def test(self):
        """Test type aliases are only searched in the debug info once"""
        self.build()
        log = self.getBuildArtifact("dwarf.log")
        self.expect("log enable dwarf lookups -f " + log)

        target, process, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift"), extra_images=['Dylib'])
        self.expect("target variable foo", substrs=["(Dylib.MyAlias)", "23"])
        self.expect("target variable bar",
                    substrs=["(Dylib.MyGenericAlias<Dylib.MyAlias>)", "42"])



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftOptimizedBoundGenericEnum(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    def test(self):
        """Test the bound generic enum types in "optimized" code."""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(self,
            'break one', lldb.SBFileSpec('main.swift'))
        bkpt_two = target.BreakpointCreateBySourceRegex(
            'break two', lldb.SBFileSpec('main.swift'))
        self.assertGreater(bkpt_two.GetNumLocations(), 0)


        var_self = self.frame().FindVariable("self")
        # FIXME, this fails with a data extractor error.
        lldbutil.check_variable(self, var_self, False, value=None)
        lldbutil.continue_to_breakpoint(process, bkpt_two)
        var_self = self.frame().FindVariable("self")
        lldbutil.check_variable(self, var_self, True, value="success")



# TestSimpleExpressions.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests simple swift expressions
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import sys
import unittest2


class TestSimpleSwiftExpressions(TestBase):
    @swiftTest
    def test_simple_swift_expressions(self):
        """Tests that we can run simple Swift expressions correctly"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        # Test that parse errors give a correct result:
        value_obj = self.frame().EvaluateExpression(
            "iff is_five === 5 { return is_five")
        error = value_obj.GetError()

        # Test simple math with constants

        lldbutil.check_expression(self, self.frame(), "5 + 6", "11", use_summary=False)
        lldbutil.check_expression(self, self.frame(), "is_five + is_six", "11", use_summary=False)
        lldbutil.check_expression(self, self.frame(),
            "if (1 == 1) { return is_five + is_six }",
            "11",
            use_summary=False)

        # Test boolean operations with simple variables:
        # Bool's are currently enums, so their value is actually in the value.
        lldbutil.check_expression(self, self.frame(), "is_eleven == is_five + is_six", "true")

        # Try a slightly more complex container for our expression:
        lldbutil.check_expression(self, self.frame(),
            "if is_five == 5 { return is_five + is_six } else { return is_five }",
            "11",
            use_summary=False)

        # Make sure we get an error if we don't give homogenous return types:
        bool_or_int = self.frame().EvaluateExpression(
            "if is_five == 5 { return is_five + is_six } else { return false }")
        self.assertTrue(
            bool_or_int.IsValid(),
            "if is_five == 5 { return is_five + is_six } else { return false } is invalid")

        # Make sure we get the correct branch of a complex result expression:
        lldbutil.check_expression(self, self.frame(),
            "if is_five == 6 {return is_five} else if is_six == 5 {return is_six} ; is_eleven",
            "11",
            use_summary=False)

        # Make sure we can access globals:
        # Commented out till we resolve <rdar://problem/15695494> Accessing global variables causes LLVM ERROR and exit...
        # lldbutil.check_expression(self, self.frame(), "my_global", "30", use_summary=True)

        # Non-simple names:
        # Note: python 2 and python 3 have different default encodings.
        # This can be removed once python 2 is gone entirely.
        if sys.version_info.major == 2:
            lldbutil.check_expression(self, self.frame(),
                u"\u20ac_varname".encode("utf-8"),
                "5",
                use_summary=False)
        else:
            lldbutil.check_expression(self, self.frame(),
                u"\u20ac_varname",
                "5",
                use_summary=False)

        # See if we can do the same manipulations with tuples:
        # Commented out due to: <rdar://problem/15476525> Expressions with
        # tuple elements assert
        lldbutil.check_expression(self, self.frame(), "a_tuple.0 + a_tuple.1", "11", use_summary=False)

        # See if we can do some manipulations with dicts:
        lldbutil.check_expression(self, self.frame(),
            'str_int_dict["five"]! + str_int_dict["six"]!',
            "11",
            use_summary=False)
        lldbutil.check_expression(self, self.frame(),
            'int_str_dict[Int(is_five + is_six)]!',
            '"eleven"')

        # Commented out, touching the dict twice causes it to die, probably the same problem
        # as <rdar://problem/15306399>
        lldbutil.check_expression(self, self.frame(),
            'str_int_dict["five"] = 6; str_int_dict["five"]! + str_int_dict["six"]!',
            "12",
            use_summary=False)

        # See if we can use a switch statement in an expression:
        lldbutil.check_expression(self, self.frame(),
            "switch is_five { case 0..<6: return 1; case 7..<11: return 2; case _: return 4; }; 3;",
            "1",
            use_summary=False)

        # These ones are int-convertible and Equatable so we can do some things
        # with them anyway:
        lldbutil.check_expression(self, self.frame(), "enum_eleven", "Eleven", use_summary=False)
        lldbutil.check_expression(self, self.frame(), "enum_eleven == SomeValues.Eleven", "true", use_summary=True)
        lldbutil.check_expression(self, self.frame(),
            "SomeValues.Five.toInt() + SomeValues.Six.toInt()",
            "11",
            use_summary=False)
        lldbutil.check_expression(self, self.frame(),
                                  "enum_eleven = .Five; return enum_eleven == .Five", "true", use_summary=True)

        # Test expressions with a simple object:
        lldbutil.check_expression(self, self.frame(), "a_obj.x", "6", use_summary=False)

        # Should not have to make a second object here. This is another
        # side-effect of <rdar://problem/15306399>
        lldbutil.check_expression(self, self.frame(), "a_nother_obj.y", "6.5", use_summary=False)

        # Test expressions with a struct:
        lldbutil.check_expression(self, self.frame(), "b_struct.b_int", "5", use_summary=False)

        # Test expression with Chars and strings:

        # Commented out due to <rdar://problem/16856379>
        #self.check_expression ("a_char", "U+0061 U+0000 u'a'")

        # Interpolated strings and string addition:
        lldbutil.check_expression(self, self.frame(),
            '"Five: \(is_five) " + "Six: \(is_six)"',
                                  '"Five: 5 Six: 6"', use_summary=True)

        # Next let's try some simple array accesses:
        lldbutil.check_expression(self, self.frame(), "an_int_array[0]", "5", use_summary=False)

        # Test expression with read-only variables:
        lldbutil.check_expression(self, self.frame(), "b_struct.b_read_only == 5", "true", use_summary=True)
        failed_value = self.frame().EvaluateExpression(
            "b_struct.b_read_only = 34")
        self.assertTrue(failed_value.IsValid(),
                        "Get something back from the evaluation.")
        self.assertTrue(failed_value.GetError().Success()
                        == False, "But it is an error.")

        # Check a simple value in a struct:
        lldbutil.check_expression(self, self.frame(), "b_struct_2.b_int", "20", use_summary=False)

        # Make sure this works for properties in extensions as well:
        lldbutil.check_expression(self, self.frame(), "b_struct_2.b_float", "20.5", use_summary=False)

        # Here are a few tests of making variables in expressions:
        lldbutil.check_expression(self, self.frame(),
            "var enum_six : SomeValues = SomeValues.Six; return enum_six == .Six", "true")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class SwiftAddressExpressionTest(TestBase):
    @swiftTest
    @skipIfLinux # rdar://125497260
    def test(self):
        """Test that you can use register names in image lookup in a swift frame."""
        self.build()
        (target, process, thread, breakpoint) = lldbutil.run_to_source_breakpoint(self, 
                "break here to check image lookup", lldb.SBFileSpec("main.swift"))
        # I don't want to be too specific in what we print for image lookup,
        # we're testing that the address expression for the pc worked.
        self.expect("image lookup -va $pc", substrs=["doSomething"])
        self.expect("image lookup -va $pc+4", substrs=["doSomething"])
        



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.async_dialect
import mlir.dialects.async_dialect.passes
from mlir.passmanager import *


def run(f):
    print("\nTEST:", f.__name__)
    f()


def testAsyncPass():
    with Context() as context:
        PassManager.parse("any(async-to-async-runtime)")
    print("SUCCESS")


# CHECK-LABEL: testAsyncPass
#       CHECK: SUCCESS
run(testAsyncPass)



"""
Test that --allow-jit=false does disallow JITting:
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestAllowJIT(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_allow_jit_expr_command(self):
        """Test the --allow-jit command line flag"""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.expr_cmd_test()

    def test_allow_jit_options(self):
        """Test the SetAllowJIT SBExpressionOption setting"""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.expr_options_test()

    def expr_cmd_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        frame = thread.GetFrameAtIndex(0)

        # First make sure we can call the function with
        interp = self.dbg.GetCommandInterpreter()
        self.expect("expr --allow-jit 1 -- call_me(10)", substrs=["(int) $", "= 18"])
        # Now make sure it fails with the "can't IR interpret message" if allow-jit is false:
        self.expect(
            "expr --allow-jit 0 -- call_me(10)",
            error=True,
            substrs=["Can't evaluate the expression without a running target"],
        )

    def expr_options_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        frame = thread.GetFrameAtIndex(0)

        # First make sure we can call the function with the default option set.
        options = lldb.SBExpressionOptions()
        # Check that the default is to allow JIT:
        self.assertEqual(options.GetAllowJIT(), True, "Default is true")

        # Now use the options:
        result = frame.EvaluateExpression("call_me(10)", options)
        self.assertSuccess(result.GetError())
        self.assertEqual(result.GetValueAsSigned(), 18, "got the right value.")

        # Now disallow JIT and make sure it fails:
        options.SetAllowJIT(False)
        # Check that we got the right value:
        self.assertEqual(
            options.GetAllowJIT(), False, "Got False after setting to False"
        )

        # Again use it and ensure we fail:
        result = frame.EvaluateExpression("call_me(10)", options)
        self.assertTrue(result.GetError().Fail(), "expression failed with no JIT")
        self.assertIn(
            "Can't evaluate the expression without a running target",
            result.GetError().GetCString(),
            "Got right error",
        )

        # Finally set the allow JIT value back to true and make sure that works:
        options.SetAllowJIT(True)
        self.assertEqual(options.GetAllowJIT(), True, "Set back to True correctly")

        # And again, make sure this works:
        result = frame.EvaluateExpression("call_me(10)", options)
        self.assertSuccess(result.GetError())
        self.assertEqual(result.GetValueAsSigned(), 18, "got the right value.")

    def test_allow_jit_with_top_level(self):
        """Test combined --allow-jit and --top-level flags"""
        # Can't force interpreting for top-level expressions which are always
        # injected.
        self.expect(
            "expr --allow-jit false --top-level -- int i;",
            error=True,
            substrs=["Can't disable JIT compilation for top-level expressions."],
        )

        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.c")
        )
        # Allowing JITing for top-level expressions is redundant but should work.
        self.expect(
            "expr --allow-jit true --top-level -- int top_level_f() { return 2; }"
        )
        # Make sure we actually declared a working top-level function.
        self.expect_expr("top_level_f()", result_value="2")



"""
Test that we can print and call closures passed in various contexts
"""

from __future__ import print_function


import os
import time
import re
import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestPassedClosures(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(bugnumber="rdar://31816998")
    def test_static_closure_type(self):
        """This tests that we can print a closure with statically known return type."""
        self.build()
        self.static_type(False)

    @expectedFailureAll(bugnumber="rdar://31816998")
    def test_static_closure_call(self):
        """This tests that we can call a closure with statically known return type."""
        self.build()
        self.static_type(True)

    @expectedFailureAll(bugnumber="rdar://31816998")
    def test_generic_closure_type(self):
        """This tests that we can print a closure with generic return type."""
        self.build()
        self.generic_type(False)

    @expectedFailureAll(bugnumber="rdar://31816998")
    def test_generic_closure_call(self):
        """This tests that we can call a closure with generic return type."""
        self.build()
        self.generic_type(True)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

    def get_to_bkpt (self, bkpt_name):
        self.build()
        lldbutil.run_to_source_breakpoint(self, bkpt_name,
                                          lldb.SBFileSpec('main.swift'))

    def static_type(self, test_call):

        self.get_to_bkpt("break here for static type")
        opts = lldb.SBExpressionOptions()

        if not test_call:
            # First see that we can print the function we were passed:
            result = self.frame().EvaluateExpression("fn", opts)
            error = result.GetError()
            self.assertSuccess(error, "'fn' failed")
            self.assertTrue("() -> Swift.Int" in result.GetValue(), "Got the function name wrong: %s."%(result.GetValue()))
            self.assertTrue("() -> Swift.Int" in result.GetTypeName(), "Got the function type wrong: %s."%(result.GetTypeName()))
        
        if test_call:
            # Now see that we can call it:
            result = self.frame().EvaluateExpression("fn()", opts)
            error.result.GetError()
            self.assertSuccess(error, "'fn()' failed")
            self.assertTrue(result.GetValue() == "3", "Got the wrong value: %s"%(result.GetValue()))

    def generic_type(self, test_call):
        self.get_to_bkpt("break here for generic type")
        opts = lldb.SBExpressionOptions()

        if not test_call:
            # First see that we can print the function we were passed:
            result = self.frame().EvaluateExpression("fn", opts)
            error = result.GetError()
            self.assertSuccess(error, "'fn' failed")
            self.assertTrue("() -> A" in result.GetValue(), "Got the function name wrong: %s."%(result.GetValue()))
            self.assertTrue("() -> A" in result.GetTypeName(), "Got the function type wrong: %s."%(result.GetTypeName()))
        
        if test_call:
            # Now see that we can call it:
            result = self.frame().EvaluateExpression("fn()", opts)
            error.result.GetError()
            self.assertSuccess(error, "'fn()' failed")
            self.assertTrue(result.GetValue() == "3", "Got the wrong value: %s"%(result.GetValue()))






"""
Test some lldb command abbreviations and aliases for proper resolution.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AbbreviationsTestCase(TestBase):
    @no_debug_info_test
    def test_command_abbreviations_and_aliases(self):
        command_interpreter = self.dbg.GetCommandInterpreter()
        self.assertTrue(command_interpreter, VALID_COMMAND_INTERPRETER)
        result = lldb.SBCommandReturnObject()

        # Check that abbreviations are expanded to the full command.
        command_interpreter.ResolveCommand("ap script", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual("apropos script", result.GetOutput())

        command_interpreter.ResolveCommand("h", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual("help", result.GetOutput())

        # Check resolution of abbreviations for multi-word commands.
        command_interpreter.ResolveCommand("lo li", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual("log list", result.GetOutput())

        command_interpreter.ResolveCommand("br s", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual("breakpoint set", result.GetOutput())

        # Try an ambiguous abbreviation.
        # "pl" could be "platform" or "plugin".
        command_interpreter.ResolveCommand("pl", result)
        self.assertFalse(result.Succeeded())
        self.assertTrue(result.GetError().startswith("Ambiguous command"))

        # Make sure an unabbreviated command is not mangled.
        command_interpreter.ResolveCommand(
            "breakpoint set --name main --line 123", result
        )
        self.assertTrue(result.Succeeded())
        self.assertEqual("breakpoint set --name main --line 123", result.GetOutput())

        # Create some aliases.
        self.runCmd("com a alias com al")
        self.runCmd("alias gurp help")

        # Check that an alias is replaced with the actual command
        command_interpreter.ResolveCommand("gurp target create", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual("help target create", result.GetOutput())

        # Delete the alias and make sure it no longer has an effect.
        self.runCmd("com u gurp")
        command_interpreter.ResolveCommand("gurp", result)
        self.assertFalse(result.Succeeded())

        # Check aliases with text replacement.
        self.runCmd("alias pltty process launch -s -o %1 -e %1")
        command_interpreter.ResolveCommand("pltty /dev/tty0", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual(
            "process launch -s -o /dev/tty0 -e /dev/tty0", result.GetOutput()
        )

        self.runCmd("alias xyzzy breakpoint set -n %1 -l %2")
        command_interpreter.ResolveCommand("xyzzy main 123", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual("breakpoint set -n main -l 123", result.GetOutput().strip())

        # And again, without enough parameters.
        command_interpreter.ResolveCommand("xyzzy main", result)
        self.assertFalse(result.Succeeded())

        # Check a command that wants the raw input.
        command_interpreter.ResolveCommand(r"""sc print("\n\n\tHello!\n")""", result)
        self.assertTrue(result.Succeeded())
        self.assertEqual(r"""script print("\n\n\tHello!\n")""", result.GetOutput())

        # Prompt changing stuff should be tested, but this doesn't seem like the
        # right test to do it in.  It has nothing to do with aliases or abbreviations.
        # self.runCmd("com sou ./change_prompt.lldb")
        # self.expect("settings show prompt",
        #            startstr = 'prompt (string) = "[with-three-trailing-spaces]   "')
        # self.runCmd("settings clear prompt")
        # self.expect("settings show prompt",
        #            startstr = 'prompt (string) = "(lldb) "')
        # self.runCmd("se se prompt 'Sycamore> '")
        # self.expect("se sh prompt",
        #            startstr = 'prompt (string) = "Sycamore> "')
        # self.runCmd("se cl prompt")
        # self.expect("set sh prompt",
        #            startstr = 'prompt (string) = "(lldb) "')



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftErrorHandlingMissingTypes(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @swiftTest
    @skipIf(setting=('symbols.use-swift-clangimporter', 'true'))
    def test(self):
        """Test that errors are surfaced"""
        self.build()
        self.expect('settings set symbols.use-swift-clangimporter false')
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift"))
        frame = thread.frames[0]
        var_object = frame.FindVariable("object", lldb.eNoDynamicValues)
        val = var_object.GetChildAtIndex(1)
        # FIXME: Should be True, for now it's just a string
        self.assertFalse(val.GetError().Fail())
        self.expect('v object',
                    substrs=['missing debug info for Clang type', 'FromC'])
        self.expect('v enm',
                    substrs=['missing debug info for Clang type', 'ImportedEnum'])



"""Test that lldb works correctly on compile units form different languages."""


import re
import lldb
from lldbsuite.test.lldbtest import *


class MixedLanguagesTestCase(TestBase):
    def test_language_of_frame(self):
        """Test that the language defaults to the language of the current frame."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Execute the cleanup function during test case tear down
        # to restore the frame format.
        def cleanup():
            self.runCmd(
                "settings set frame-format %s" % self.format_string, check=False
            )

        self.addTearDownHook(cleanup)
        self.runCmd("settings show frame-format")
        m = re.match('^frame-format \(format-string\) = "(.*)"$', self.res.GetOutput())
        self.assertTrue(m, "Bad settings string")
        self.format_string = m.group(1)

        # Change the default format to print the language.
        format_string = "frame #${frame.index}: ${frame.pc}{ ${module.file.basename}\`${function.name}{${function.pc-offset}}}{, lang=${language}}\n"
        self.runCmd("settings set frame-format %s" % format_string)
        self.expect(
            "settings show frame-format",
            SETTING_MSG("frame-format"),
            substrs=[format_string],
        )

        # Run to BP at main (in main.c) and test that the language is C.
        self.runCmd("breakpoint set -n main")
        self.runCmd("run")
        self.expect("thread backtrace", substrs=["`main", "lang=c"])
        # Make sure evaluation of C++11 fails.
        self.expect("expr foo != nullptr", error=True, startstr="error")

        # Run to BP at foo (in foo.cpp) and test that the language is C++.
        self.runCmd("breakpoint set -n foo")
        self.runCmd("continue")
        self.expect("thread backtrace", substrs=["`::foo()", "lang=c++"])
        # Make sure we can evaluate an expression requiring C++11
        # (note: C++11 is enabled by default for C++).
        self.expect("expr foo != nullptr", patterns=["true"])



# TestSwiftStaticArchiveTwoSwiftmodules.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2

class TestSwiftStaticArchiveTwoSwiftmodules(TestBase):
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        self.build()
        exe_name = "a.out"
        exe = self.getBuildArtifact(exe_name)

        # Create the target.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints.
        foo_breakpoint = target.BreakpointCreateBySourceRegex(
            'break here', lldb.SBFileSpec('Foo.swift'))
        bar_breakpoint = target.BreakpointCreateBySourceRegex(
            'break here', lldb.SBFileSpec('Bar.swift'))
        self.assertTrue(foo_breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT)
        self.assertTrue(bar_breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT)

        # Launch.
        process = target.LaunchSimple(None, None, os.getcwd())

        # This test tests that the search paths from all swiftmodules
        # that are part of the main binary are honored.
        self.expect("fr var foo", "expected result", substrs=["23"])
        self.expect("expression foo", "expected result", substrs=["$R0", "i", "23"])
        process.Continue()
        self.expect("fr var bar", "expected result", substrs=["42"])
        self.expect("expression bar", "expected result", substrs=["j", "42"])



"""
Test controlling `expression` result variables are persistent.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @swiftTest
    def test_enable_persistent_result(self):
        """Test explicitly enabling result variables persistence."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        self.expect("expression --persistent-result on -- i", startstr="(Int) $R0 = 30")
        # Verify the lifetime of $R0 extends beyond the `expression` it was created in.
        self.expect("expression $R0", startstr="(Int) $R1 = 30")

    @swiftTest
    def test_disable_persistent_result(self):
        """Test explicitly disabling persistent result variables."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        self.expect("expression --persistent-result off -- i", startstr="(Int) 30")
        # Verify a persistent result was not silently created.
        self.expect("expression $R0", error=True)

    @swiftTest
    def test_expression_persists_result(self):
        """Test `expression`'s default behavior is to persist a result variable."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        self.expect("expression i", startstr="(Int) $R0 = 30")
        self.expect("expression $R0", startstr="(Int) $R1 = 30")

    @swiftTest
    def test_p_does_not_persist_results(self):
        """Test `p` does not persist a result variable."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        self.expect("p i", startstr="(Int) 30")
        self.expect("p $R0", error=True)



#!/usr/bin/env python
"""
Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
See https://llvm.org/LICENSE.txt for license information.
SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

Provides classes used by the test results reporting infrastructure
within the LLDB test suite.


Tests the process_control module.
"""


# System imports.
import os
import os.path
import unittest
import sys
import threading

# Our imports.
from test_runner import process_control


class TestInferiorDriver(process_control.ProcessDriver):
    def __init__(self, soft_terminate_timeout=None):
        super(TestInferiorDriver, self).__init__(
            soft_terminate_timeout=soft_terminate_timeout
        )
        self.started_event = threading.Event()
        self.started_event.clear()

        self.completed_event = threading.Event()
        self.completed_event.clear()

        self.was_timeout = False
        self.returncode = None
        self.output = None

    def write(self, content):
        # We'll swallow this to keep tests non-noisy.
        # Uncomment the following line if you want to see it.
        # sys.stdout.write(content)
        pass

    def on_process_started(self):
        self.started_event.set()

    def on_process_exited(self, command, output, was_timeout, exit_status):
        self.returncode = exit_status
        self.was_timeout = was_timeout
        self.output = output
        self.returncode = exit_status
        self.completed_event.set()


class ProcessControlTests(unittest.TestCase):
    @classmethod
    def _suppress_soft_terminate(cls, command):
        # Do the right thing for your platform here.
        # Right now only POSIX-y systems are reporting
        # soft terminate support, so this is set up for
        # those.
        helper = process_control.ProcessHelper.process_helper()
        signals = helper.soft_terminate_signals()
        if signals is not None:
            for signum in helper.soft_terminate_signals():
                command.extend(["--ignore-signal", str(signum)])

    @classmethod
    def inferior_command(cls, ignore_soft_terminate=False, options=None):
        # Base command.
        script_name = "{}/inferior.py".format(os.path.dirname(__file__))
        if not os.path.exists(script_name):
            raise Exception(
                "test inferior python script not found: {}".format(script_name)
            )
        command = [sys.executable, script_name]

        if ignore_soft_terminate:
            cls._suppress_soft_terminate(command)

        # Handle options as string or list.
        if isinstance(options, str):
            command.extend(options.split())
        elif isinstance(options, list):
            command.extend(options)

        # Return full command.
        return command


class ProcessControlNoTimeoutTests(ProcessControlTests):
    """Tests the process_control module."""

    def test_run_completes(self):
        """Test that running completes and gets expected stdout/stderr."""
        driver = TestInferiorDriver()
        driver.run_command(self.inferior_command())
        self.assertTrue(driver.completed_event.wait(5), "process failed to complete")
        self.assertEqual(driver.returncode, 0, "return code does not match")

    def test_run_completes_with_code(self):
        """Test that running completes and gets expected stdout/stderr."""
        driver = TestInferiorDriver()
        driver.run_command(self.inferior_command(options="-r10"))
        self.assertTrue(driver.completed_event.wait(5), "process failed to complete")
        self.assertEqual(driver.returncode, 10, "return code does not match")


class ProcessControlTimeoutTests(ProcessControlTests):
    def test_run_completes(self):
        """Test that running completes and gets expected return code."""
        driver = TestInferiorDriver()
        timeout_seconds = 5
        driver.run_command_with_timeout(
            self.inferior_command(), "{}s".format(timeout_seconds), False
        )
        self.assertTrue(
            driver.completed_event.wait(2 * timeout_seconds),
            "process failed to complete",
        )
        self.assertEqual(driver.returncode, 0)

    def _soft_terminate_works(self, with_core):
        # Skip this test if the platform doesn't support soft ti
        helper = process_control.ProcessHelper.process_helper()
        if not helper.supports_soft_terminate():
            self.skipTest("soft terminate not supported by platform")

        driver = TestInferiorDriver()
        timeout_seconds = 5

        driver.run_command_with_timeout(
            # Sleep twice as long as the timeout interval.  This
            # should force a timeout.
            self.inferior_command(options="--sleep {}".format(timeout_seconds * 2)),
            "{}s".format(timeout_seconds),
            with_core,
        )

        # We should complete, albeit with a timeout.
        self.assertTrue(
            driver.completed_event.wait(2 * timeout_seconds),
            "process failed to complete",
        )

        # Ensure we received a timeout.
        self.assertTrue(driver.was_timeout, "expected to end with a timeout")

        self.assertTrue(
            helper.was_soft_terminate(driver.returncode, with_core),
            (
                "timeout didn't return expected returncode "
                "for soft terminate with core: {}"
            ).format(driver.returncode),
        )

    def test_soft_terminate_works_core(self):
        """Driver uses soft terminate (with core request) when process times out."""
        self._soft_terminate_works(True)

    def test_soft_terminate_works_no_core(self):
        """Driver uses soft terminate (no core request) when process times out."""
        self._soft_terminate_works(False)

    def test_hard_terminate_works(self):
        """Driver falls back to hard terminate when soft terminate is ignored."""

        driver = TestInferiorDriver(soft_terminate_timeout=2.0)
        timeout_seconds = 1

        driver.run_command_with_timeout(
            # Sleep much longer than the timeout interval,forcing a
            # timeout.  Do whatever is needed to have the inferior
            # ignore soft terminate calls.
            self.inferior_command(ignore_soft_terminate=True, options="--never-return"),
            "{}s".format(timeout_seconds),
            True,
        )

        # We should complete, albeit with a timeout.
        self.assertTrue(driver.completed_event.wait(60), "process failed to complete")

        # Ensure we received a timeout.
        self.assertTrue(driver.was_timeout, "expected to end with a timeout")

        helper = process_control.ProcessHelper.process_helper()
        self.assertTrue(
            helper.was_hard_terminate(driver.returncode),
            (
                "timeout didn't return expected returncode "
                "for hard teriminate: {} ({})"
            ).format(driver.returncode, driver.output),
        )

    def test_inferior_exits_with_live_child_shared_handles(self):
        """inferior exit detected when inferior children are live with shared
        stdout/stderr handles.
        """
        # Requires review D13362 or equivalent to be implemented.
        self.skipTest("http://reviews.llvm.org/D13362")

        driver = TestInferiorDriver()

        # Create the inferior (I1), and instruct it to create a child (C1)
        # that shares the stdout/stderr handles with the inferior.
        # C1 will then loop forever.
        driver.run_command_with_timeout(
            self.inferior_command(
                options="--launch-child-share-handles --return-code 3"
            ),
            "5s",
            False,
        )

        # We should complete without a timetout.  I1 should end
        # immediately after launching C1.
        self.assertTrue(driver.completed_event.wait(5), "process failed to complete")

        # Ensure we didn't receive a timeout.
        self.assertFalse(driver.was_timeout, "inferior should have completed normally")

        self.assertEqual(
            driver.returncode,
            3,
            "expected inferior process to end with expected returncode",
        )


if __name__ == "__main__":
    unittest.main()



"""Test binaries with delay-init dependencies."""

import subprocess
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDelayInitDependencies(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    @skipIf(macos_version=["<", "15.0"])
    def test_delay_init_dependency(self):
        TestBase.setUp(self)
        out = subprocess.run(
            ["xcrun", "ld", "-delay_library"],
            universal_newlines=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        if "delay_library missing" not in out.stderr:
            self.skipTest(
                "Skipped because the linker doesn't know about -delay_library"
            )
        self.build()
        main_source = "main.c"
        exe = self.getBuildArtifact("a.out")
        lib = self.getBuildArtifact("libfoo.dylib")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # libfoo.dylib should not be in the target pre-execution
        for m in target.modules:
            self.assertNotEqual(m.GetFileSpec().GetFilename(), "libfoo.dylib")

        # This run without arguments will not load libfoo.dylib
        li = lldb.SBLaunchInfo([])
        li.SetWorkingDirectory(self.getBuildDir())
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c"), li
        )
        for m in target.modules:
            self.assertNotEqual(m.GetFileSpec().GetFilename(), "libfoo.dylib")

        process.Kill()
        self.dbg.DeleteTarget(target)

        # This run with one argument will load libfoo.dylib
        li = lldb.SBLaunchInfo([])
        li.SetWorkingDirectory(self.getBuildDir())
        li.SetArguments(["one-argument"], True)
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c"), li
        )

        found_libfoo = False
        for m in target.modules:
            if m.GetFileSpec().GetFilename() == "libfoo.dylib":
                found_libfoo = True
        self.assertTrue(found_libfoo)



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

try:
    from ..ir import *
    from ._ods_common import (
        get_op_result_or_value as _get_op_result_or_value,
        get_op_results_or_values as _get_op_results_or_values,
    )
except ImportError as e:
    raise RuntimeError("Error loading imports from extension module") from e

from typing import Optional, Sequence, Union


class CastOp:

  def __init__(
      self,
      result_type: Type,
      target: Union[Operation, Value],
      *,
      loc=None,
      ip=None,
  ):
    super().__init__(
        result_type, _get_op_result_or_value(target), loc=loc, ip=ip
    )


class ApplyPatternsOp:

  def __init__(
      self,
      target: Union[Operation, Value, OpView],
      *,
      loc=None,
      ip=None,
  ):
    operands = []
    operands.append(_get_op_result_or_value(target))
    super().__init__(
        self.build_generic(attributes={},
                           results=[],
                           operands=operands,
                           successors=None,
                           regions=None,
                           loc=loc,
                           ip=ip))
    self.regions[0].blocks.append()

  @property
  def patterns(self) -> Block:
    return self.regions[0].blocks[0]


class testGetParentOp:

  def __init__(
      self,
      result_type: Type,
      target: Union[Operation, Value],
      *,
      isolated_from_above: bool = False,
      op_name: Optional[str] = None,
      deduplicate: bool = False,
      loc=None,
      ip=None,
  ):
    super().__init__(
        result_type,
        _get_op_result_or_value(target),
        isolated_from_above=isolated_from_above,
        op_name=op_name,
        deduplicate=deduplicate,
        loc=loc,
        ip=ip,
    )


class MergeHandlesOp:
    def __init__(
        self,
        handles: Sequence[Union[Operation, Value]],
        *,
        deduplicate: bool = False,
        loc=None,
        ip=None,
    ):
        super().__init__(
            [_get_op_result_or_value(h) for h in handles],
            deduplicate=deduplicate,
            loc=loc,
            ip=ip,
        )


class ReplicateOp:
    def __init__(
        self,
        pattern: Union[Operation, Value],
        handles: Sequence[Union[Operation, Value]],
        *,
        loc=None,
        ip=None,
    ):
        super().__init__(
            [_get_op_result_or_value(h).type for h in handles],
            _get_op_result_or_value(pattern),
            [_get_op_result_or_value(h) for h in handles],
            loc=loc,
            ip=ip,
        )


class SequenceOp:
    def __init__(
        self,
        failure_propagation_mode,
        results: Sequence[Type],
        target: Union[Operation, Value, Type],
        extra_bindings: Optional[
            Union[Sequence[Value], Sequence[Type], Operation, OpView]
        ] = None,
    ):
        root = (
            _get_op_result_or_value(target)
            if isinstance(target, (Operation, Value))
            else None
        )
        root_type = root.type if not isinstance(target, Type) else target
        if not isinstance(failure_propagation_mode, Attribute):
            failure_propagation_mode_attr = IntegerAttr.get(
                IntegerType.get_signless(32), failure_propagation_mode._as_int()
            )
        else:
            failure_propagation_mode_attr = failure_propagation_mode

        if extra_bindings is None:
            extra_bindings = []
        if isinstance(extra_bindings, (Operation, OpView)):
            extra_bindings = _get_op_results_or_values(extra_bindings)

        extra_binding_types = []
        if len(extra_bindings) != 0:
            if isinstance(extra_bindings[0], Type):
                extra_binding_types = extra_bindings
                extra_bindings = []
            else:
                extra_binding_types = [v.type for v in extra_bindings]

        super().__init__(
            results_=results,
            failure_propagation_mode=failure_propagation_mode_attr,
            root=root,
            extra_bindings=extra_bindings,
        )
        self.regions[0].blocks.append(*tuple([root_type] + extra_binding_types))

    @property
    def body(self) -> Block:
        return self.regions[0].blocks[0]

    @property
    def bodyTarget(self) -> Value:
        return self.body.arguments[0]

    @property
    def bodyExtraArgs(self) -> BlockArgumentList:
        return self.body.arguments[1:]


class YieldOp:
    def __init__(
        self,
        operands: Optional[Union[Operation, Sequence[Value]]] = None,
        *,
        loc=None,
        ip=None,
    ):
        if operands is None:
            operands = []
        super().__init__(_get_op_results_or_values(operands), loc=loc, ip=ip)



"""
Test the AArch64 SVE registers dynamic resize with multiple threads.

This test assumes a minimum supported vector length (VL) of 256 bits
and will test 512 bits if possible. We refer to "vg" which is the
register shown in lldb. This is in units of 64 bits. 256 bit VL is
the same as a vg of 4.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class RegisterCommandsTestCase(TestBase):
    def get_supported_vg(self):
        # Changing VL trashes the register state, so we need to run the program
        # just to test this. Then run it again for the test.
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        main_thread_stop_line = line_number("main.c", "// Break in main thread")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", main_thread_stop_line)

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread info 1",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
        )

        # Write back the current vg to confirm read/write works at all.
        current_vg = self.match("register read vg", ["(0x[0-9]+)"])
        self.assertTrue(current_vg is not None)
        self.expect("register write vg {}".format(current_vg.group()))

        # Aka 128, 256 and 512 bit.
        supported_vg = []
        for vg in [2, 4, 8]:
            # This could mask other errors but writing vg is tested elsewhere
            # so we assume the hardware rejected the value.
            self.runCmd("register write vg {}".format(vg), check=False)
            if not self.res.GetError():
                supported_vg.append(vg)

        return supported_vg

    def check_sve_registers(self, vg_test_value):
        z_reg_size = vg_test_value * 8
        p_reg_size = int(z_reg_size / 8)

        p_value_bytes = ["0xff", "0x55", "0x11", "0x01", "0x00"]

        for i in range(32):
            s_reg_value = "s%i = 0x" % (i) + "".join(
                "{:02x}".format(i + 1) for _ in range(4)
            )

            d_reg_value = "d%i = 0x" % (i) + "".join(
                "{:02x}".format(i + 1) for _ in range(8)
            )

            v_reg_value = "v%i = 0x" % (i) + "".join(
                "{:02x}".format(i + 1) for _ in range(16)
            )

            z_reg_value = (
                "{"
                + " ".join("0x{:02x}".format(i + 1) for _ in range(z_reg_size))
                + "}"
            )

            self.expect("register read -f hex " + "s%i" % (i), substrs=[s_reg_value])

            self.expect("register read -f hex " + "d%i" % (i), substrs=[d_reg_value])

            self.expect("register read -f hex " + "v%i" % (i), substrs=[v_reg_value])

            self.expect("register read " + "z%i" % (i), substrs=[z_reg_value])

        for i in range(16):
            p_regs_value = (
                "{" + " ".join(p_value_bytes[i % 5] for _ in range(p_reg_size)) + "}"
            )
            self.expect("register read " + "p%i" % (i), substrs=[p_regs_value])

        self.expect("register read ffr", substrs=[p_regs_value])

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_sve_registers_dynamic_config(self):
        """Test AArch64 SVE registers multi-threaded dynamic resize."""

        if not self.isAArch64SVE():
            self.skipTest("SVE registers must be supported.")

        self.build()
        supported_vg = self.get_supported_vg()

        if not (2 in supported_vg and 4 in supported_vg):
            self.skipTest("Not all required SVE vector lengths are supported.")

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        main_thread_stop_line = line_number("main.c", "// Break in main thread")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", main_thread_stop_line)

        thX_break_line1 = line_number("main.c", "// Thread X breakpoint 1")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", thX_break_line1)

        thX_break_line2 = line_number("main.c", "// Thread X breakpoint 2")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", thX_break_line2)

        thY_break_line1 = line_number("main.c", "// Thread Y breakpoint 1")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", thY_break_line1)

        thY_break_line2 = line_number("main.c", "// Thread Y breakpoint 2")
        lldbutil.run_break_set_by_file_and_line(self, "main.c", thY_break_line2)

        self.runCmd("run", RUN_SUCCEEDED)

        process = self.dbg.GetSelectedTarget().GetProcess()

        self.expect(
            "thread info 1",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint"],
        )

        if 8 in supported_vg:
            self.check_sve_registers(8)
        else:
            self.check_sve_registers(4)

        self.runCmd("process continue", RUN_SUCCEEDED)

        # If we start the checks too quickly, thread 3 may not have started.
        while (process.GetNumThreads() < 3):
            pass

        for idx in range(1, process.GetNumThreads()):
            thread = process.GetThreadAtIndex(idx)
            if thread.GetStopReason() != lldb.eStopReasonBreakpoint:
                self.runCmd("thread continue %d" % (idx + 1))
                self.assertEqual(thread.GetStopReason(), lldb.eStopReasonBreakpoint)

            stopped_at_line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()

            if stopped_at_line_number == thX_break_line1:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(4)
                self.runCmd("register write vg 2")

            elif stopped_at_line_number == thY_break_line1:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(2)
                self.runCmd("register write vg 4")

        self.runCmd("thread continue 2")
        self.runCmd("thread continue 3")

        for idx in range(1, process.GetNumThreads()):
            thread = process.GetThreadAtIndex(idx)
            self.assertEqual(thread.GetStopReason(), lldb.eStopReasonBreakpoint)

            stopped_at_line_number = thread.GetFrameAtIndex(0).GetLineEntry().GetLine()

            if stopped_at_line_number == thX_break_line2:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(2)

            elif stopped_at_line_number == thY_break_line2:
                self.runCmd("thread select %d" % (idx + 1))
                self.check_sve_registers(4)



"""
Test more expression command sequences with objective-c.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FoundationTestCaseNSError(TestBase):
    @expectedFailureAll(archs=["i[3-6]86"], bugnumber="<rdar://problem/28814052>")
    def test_runtime_types(self):
        """Test commands that require runtime types"""
        self.build()
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Break here for NSString tests", lldb.SBFileSpec("main.m", False)
        )

        # Test_NSString:
        self.runCmd("thread backtrace")
        self.expect("expression [str length]", patterns=["\(NSUInteger\) \$.* ="])
        self.expect("expression str.length")
        self.expect('expression str = [NSString stringWithCString: "new"]')
        self.expect(
            'po [NSError errorWithDomain:@"Hello" code:35 userInfo:@{@"NSDescription" : @"be completed."}]',
            substrs=["Error Domain=Hello", "Code=35", "be completed."],
        )
        self.runCmd("process continue")

    @expectedFailureAll(archs=["i[3-6]86"], bugnumber="<rdar://problem/28814052>")
    def test_NSError_p(self):
        """Test that p of the result of an unknown method does require a cast."""
        self.build()
        self.target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line", lldb.SBFileSpec("main.m", False)
        )
        self.expect(
            "expression [NSError thisMethodIsntImplemented:0]",
            error=True,
            patterns=[
                "no known method",
                "cast the message send to the method's return type",
            ],
        )
        self.runCmd("process continue")



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftModuleImport(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)
    NO_DEBUG_INFO_TESTCASE = True

    @swiftTest
    def test(self):
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)
        self.expect("expression -- 0")
        self.filecheck('platform shell cat "%s"' % log, __file__)
#       CHECK: SwiftASTContextForExpressions{{.*}}Module import remark: loaded module 'a'



"""Test that lldb can create a skinny corefile, and load all available libraries correctly."""


import os
import re
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestSkinnyCorefile(TestBase):
    @skipIfOutOfTreeDebugserver  # newer debugserver required for these qMemoryRegionInfo types
    @skipIf(
        debug_info=no_match(["dsym"]),
        bugnumber="This test is looking explicitly for a dSYM",
    )
    @skipUnlessDarwin
    @skipIfRemote
    def test_lc_note(self):
        self.build()
        self.aout_exe = self.getBuildArtifact("a.out")
        self.aout_dsym = self.getBuildArtifact("a.out.dSYM")
        self.to_be_removed_dylib = self.getBuildArtifact("libto-be-removed.dylib")
        self.to_be_removed_dsym = self.getBuildArtifact("libto-be-removed.dylib.dSYM")
        self.corefile = self.getBuildArtifact("process.core")
        self.dsym_for_uuid = self.getBuildArtifact("dsym-for-uuid.sh")

        # After the corefile is created, we'll move a.out and a.out.dSYM
        # into hide.noindex and lldb will have to use the
        # LLDB_APPLE_DSYMFORUUID_EXECUTABLE script to find them.
        self.hide_dir = self.getBuildArtifact("hide.noindex")
        lldbutil.mkdir_p(self.hide_dir)
        self.hide_aout_exe = self.getBuildArtifact("hide.noindex/a.out")
        self.hide_aout_dsym = self.getBuildArtifact("hide.noindex/a.out.dSYM")

        # We can hook in our dsym-for-uuid shell script to lldb with
        # this env var instead of requiring a defaults write.
        os.environ["LLDB_APPLE_DSYMFORUUID_EXECUTABLE"] = self.dsym_for_uuid
        self.addTearDownHook(
            lambda: os.environ.pop("LLDB_APPLE_DSYMFORUUID_EXECUTABLE", None)
        )

        dwarfdump_uuid_regex = re.compile("UUID: ([-0-9a-fA-F]+) \(([^\(]+)\) .*")
        dwarfdump_cmd_output = subprocess.check_output(
            ('/usr/bin/dwarfdump --uuid "%s"' % self.aout_exe), shell=True
        ).decode("utf-8")
        aout_uuid = None
        for line in dwarfdump_cmd_output.splitlines():
            match = dwarfdump_uuid_regex.search(line)
            if match:
                aout_uuid = match.group(1)
        self.assertNotEqual(aout_uuid, None, "Could not get uuid of built a.out")

        ###  Create our dsym-for-uuid shell script which returns self.hide_aout_exe.
        shell_cmds = [
            "#! /bin/sh",
            "# the last argument is the uuid",
            "while [ $# -gt 1 ]",
            "do",
            "  shift",
            "done",
            "ret=0",
            'echo "<?xml version=\\"1.0\\" encoding=\\"UTF-8\\"?>"',
            'echo "<!DOCTYPE plist PUBLIC \\"-//Apple//DTD PLIST 1.0//EN\\" \\"http://www.apple.com/DTDs/PropertyList-1.0.dtd\\">"',
            'echo "<plist version=\\"1.0\\">"',
            "",
            'if [ "$1" = "%s" ]' % aout_uuid,
            "then",
            "  uuid=%s" % aout_uuid,
            "  bin=%s" % self.hide_aout_exe,
            "  dsym=%s.dSYM/Contents/Resources/DWARF/%s"
            % (self.hide_aout_exe, os.path.basename(self.hide_aout_exe)),
            "fi",
            'if [ -z "$uuid" -o -z "$bin" -o ! -f "$bin" ]',
            "then",
            '  echo "<key>DBGError</key><string>not found</string>"',
            '  echo "</plist>"',
            "  exit 1",
            "fi",
            'echo "<dict><key>$uuid</key><dict>"',
            "",
            'echo "<key>DBGArchitecture</key><string>x86_64</string>"',
            'echo "<key>DBGDSYMPath</key><string>$dsym</string>"',
            'echo "<key>DBGSymbolRichExecutable</key><string>$bin</string>"',
            'echo "</dict></dict></plist>"',
            "exit $ret",
        ]

        with open(self.dsym_for_uuid, "w") as writer:
            for l in shell_cmds:
                writer.write(l + "\n")

        os.chmod(self.dsym_for_uuid, 0o755)

        # Launch a live process with a.out, libto-be-removed.dylib,
        # libpresent.dylib all in their original locations, create
        # a corefile at the breakpoint.
        (target, process, t, bp) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("present.c")
        )

        self.assertTrue(process.IsValid())

        if self.TraceOn():
            self.runCmd("bt")
            self.runCmd("image list")

        self.runCmd("process save-core " + self.corefile)
        process.Kill()
        target.Clear()

        # Move the main binary and its dSYM into the hide.noindex
        # directory.  Now the only way lldb can find them is with
        # the LLDB_APPLE_DSYMFORUUID_EXECUTABLE shell script -
        # so we're testing that this dSYM discovery method works.
        os.rename(self.aout_exe, self.hide_aout_exe)
        os.rename(self.aout_dsym, self.hide_aout_dsym)

        # Completely remove the libto-be-removed.dylib, so we're
        # testing that lldb handles an unavailable binary correctly,
        # and non-dirty memory from this binary (e.g. the executing
        # instructions) are NOT included in the corefile.
        os.unlink(self.to_be_removed_dylib)
        shutil.rmtree(self.to_be_removed_dsym)

        # Now load the corefile
        self.target = self.dbg.CreateTarget("")
        self.process = self.target.LoadCore(self.corefile)
        self.assertTrue(self.process.IsValid())
        if self.TraceOn():
            self.runCmd("image list")
            self.runCmd("bt")

        self.assertTrue(self.process.IsValid())
        self.assertTrue(self.process.GetSelectedThread().IsValid())

        # f0 is present() in libpresent.dylib
        f0 = self.process.GetSelectedThread().GetFrameAtIndex(0)
        to_be_removed_dirty_data = f0.FindVariable("to_be_removed_dirty_data")
        self.assertEqual(to_be_removed_dirty_data.GetValueAsUnsigned(), 20)

        present_heap_buf = f0.FindVariable("present_heap_buf")
        self.assertIn("have ints 5 20 20 5", present_heap_buf.GetSummary())

        # f1 is to_be_removed() in libto-be-removed.dylib
        # it has been removed since the corefile was created,
        # and the instructions for this frame should NOT be included
        # in the corefile.  They were not dirty pages.
        f1 = self.process.GetSelectedThread().GetFrameAtIndex(1)
        err = lldb.SBError()
        uint = self.process.ReadUnsignedFromMemory(f1.GetPC(), 4, err)
        self.assertTrue(err.Fail())

        # TODO Future testing could check that read-only constant data
        # (main_const_data, present_const_data) can be read both as an
        # SBValue and in an expression -- which means lldb needs to read
        # them out of the binaries, they are not present in the corefile.
        # And checking file-scope dirty data (main_dirty_data,
        # present_dirty_data) the same way would be good, instead of just
        # checking the heap and stack like are being done right now.



"""
Make sure if we have two classes with the same base name the
dynamic value calculator doesn't confuse them
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class DynamicValueSameBaseTestCase(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_same_basename_this(self):
        """Test that the we use the full name to resolve dynamic types."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        self.sample_test()

    def sample_test(self):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Break here to get started", self.main_source_file
        )

        # Set breakpoints in the two class methods and run to them:
        namesp_bkpt = target.BreakpointCreateBySourceRegex(
            "namesp function did something.", self.main_source_file
        )
        self.assertEqual(
            namesp_bkpt.GetNumLocations(), 1, "Namespace breakpoint invalid"
        )

        virtual_bkpt = target.BreakpointCreateBySourceRegex(
            "Virtual function did something.", self.main_source_file
        )
        self.assertEqual(
            virtual_bkpt.GetNumLocations(), 1, "Virtual breakpoint invalid"
        )

        threads = lldbutil.continue_to_breakpoint(process, namesp_bkpt)
        self.assertEqual(len(threads), 1, "Didn't stop at namespace breakpoint")

        frame = threads[0].frame[0]
        namesp_this = frame.FindVariable("this", lldb.eDynamicCanRunTarget)
        # Clang specifies the type of this as "T *", gcc as "T * const". This
        # erases the difference.
        namesp_type = namesp_this.GetType().GetUnqualifiedType()
        self.assertEqual(
            namesp_type.GetName(),
            "namesp::Virtual *",
            "Didn't get the right dynamic type",
        )

        threads = lldbutil.continue_to_breakpoint(process, virtual_bkpt)
        self.assertEqual(len(threads), 1, "Didn't stop at virtual breakpoint")

        frame = threads[0].frame[0]
        virtual_this = frame.FindVariable("this", lldb.eDynamicCanRunTarget)
        virtual_type = virtual_this.GetType().GetUnqualifiedType()
        self.assertEqual(
            virtual_type.GetName(), "Virtual *", "Didn't get the right dynamic type"
        )



"""
Tests standard library iterators.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        iter_type = "std::move_iterator<std::__wrap_iter<int *> >"

        self.expect_expr("move_begin", result_type=iter_type)
        self.expect_expr("move_begin[0]", result_type="int", result_value="1")

        self.expect_expr("move_begin + 3 == move_end", result_value="true")

        self.expect("expr move_begin++")
        self.expect_expr("move_begin + 2 == move_end", result_value="true")
        self.expect("expr move_begin--")
        self.expect_expr("move_begin + 3 == move_end", result_value="true")



"""
Test the 'memory read' command.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class MemoryReadTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def build_run_stop(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

    def test_memory_read_c_string(self):
        """Test that reading memory as a c string respects the size limit given
        and warns if the null terminator is missing."""
        self.build_run_stop()

        # The size here is the size in memory so it includes the null terminator.
        cmd = 'memory read --format "c-string" --size {} &my_string'

        # Size matches the size of the array.
        self.expect(cmd.format(8), substrs=['"abcdefg"'])

        # If size would take us past the terminator we stop at the terminator.
        self.expect(cmd.format(10), substrs=['"abcdefg"'])

        # Size 3 means 2 chars and a terminator. So we print 2 chars but warn because
        # the third isn't 0 as expected.
        self.expect(cmd.format(3), substrs=['"ab"'])
        self.assertRegex(
            self.res.GetError(),
            "unable to find a NULL terminated string at 0x[0-9A-Fa-f]+."
            " Consider increasing the maximum read length.",
        )

    def test_memory_read(self):
        """Test the 'memory read' command with plain and vector formats."""
        self.build_run_stop()

        # (lldb) memory read -f d -c 1 `&argc`
        # 0x7fff5fbff9a0: 1
        self.runCmd("memory read -f d -c 1 `&argc`")

        # Find the starting address for variable 'argc' to verify later that the
        # '--format uint32_t[] --size 4 --count 4' option increments the address
        # correctly.
        line = self.res.GetOutput().splitlines()[0]
        items = line.split(":")
        address = int(items[0], 0)
        argc = int(items[1], 0)
        self.assertGreater(address, 0)
        self.assertEqual(argc, 1)

        # (lldb) memory read --format uint32_t[] --size 4 --count 4 `&argc`
        # 0x7fff5fbff9a0: {0x00000001}
        # 0x7fff5fbff9a4: {0x00000000}
        # 0x7fff5fbff9a8: {0x0ec0bf27}
        # 0x7fff5fbff9ac: {0x215db505}
        self.runCmd("memory read --format uint32_t[] --size 4 --count 4 `&argc`")
        lines = self.res.GetOutput().splitlines()
        for i in range(4):
            if i == 0:
                # Verify that the printout for argc is correct.
                self.assertEqual(argc, int(lines[i].split(":")[1].strip(" {}"), 0))
            addr = int(lines[i].split(":")[0], 0)
            # Verify that the printout for addr is incremented correctly.
            self.assertEqual(addr, (address + i * 4))

        # (lldb) memory read --format char[] --size 7 --count 1 `&my_string`
        # 0x7fff5fbff990: {abcdefg}
        self.expect(
            "memory read --format char[] --size 7 --count 1 `&my_string`",
            substrs=["abcdefg"],
        )

        # (lldb) memory read --format 'hex float' --size 16 `&argc`
        # 0x7fff5fbff5b0: error: unsupported byte size (16) for hex float
        # format
        self.expect(
            "memory read --format 'hex float' --size 16 `&argc`",
            substrs=["unsupported byte size (16) for hex float format"],
        )

        self.expect(
            "memory read --format 'float' --count 1 --size 8 `&my_double`",
            substrs=["1234."],
        )

        # (lldb) memory read --format 'float' --count 1 --size 20 `&my_double`
        # 0x7fff5fbff598: error: unsupported byte size (20) for float format
        self.expect(
            "memory read --format 'float' --count 1 --size 20 `&my_double`",
            substrs=["unsupported byte size (20) for float format"],
        )

        self.expect(
            "memory read --type int --count 5 `&my_ints[0]`",
            substrs=["(int) 0x", "2", "4", "6", "8", "10"],
        )

        self.expect(
            "memory read --type int --count 5 --format hex `&my_ints[0]`",
            substrs=["(int) 0x", "0x", "0a"],
        )

        self.expect(
            "memory read --type int --count 5 --offset 5 `&my_ints[0]`",
            substrs=["(int) 0x", "12", "14", "16", "18", "20"],
        )

        # the gdb format specifier and the size in characters for
        # the returned values including the 0x prefix.
        variations = [["b", 4], ["h", 6], ["w", 10], ["g", 18]]
        for v in variations:
            formatter = v[0]
            expected_object_length = v[1]
            self.runCmd("memory read --gdb-format 4%s &my_uint64s" % formatter)
            lines = self.res.GetOutput().splitlines()
            objects_read = []
            for l in lines:
                objects_read.extend(l.split(":")[1].split())
            # Check that we got back 4 0x0000 etc bytes
            for o in objects_read:
                self.assertEqual(len(o), expected_object_length)
            self.assertEqual(len(objects_read), 4)

    def test_memory_read_file(self):
        self.build_run_stop()
        res = lldb.SBCommandReturnObject()
        self.ci.HandleCommand("memory read -f d -c 1 `&argc`", res)
        self.assertTrue(res.Succeeded(), "memory read failed:" + res.GetError())

        # Record golden output.
        golden_output = res.GetOutput()

        memory_read_file = self.getBuildArtifact("memory-read-output")

        def check_file_content(expected):
            with open(memory_read_file) as f:
                lines = f.readlines()
                lines = [s.strip() for s in lines]
                expected = [s.strip() for s in expected]
                self.assertEqual(lines, expected)

        # Sanity check.
        self.runCmd("memory read -f d -c 1 -o '{}' `&argc`".format(memory_read_file))
        check_file_content([golden_output])

        # Write some garbage to the file.
        with open(memory_read_file, "w") as f:
            f.write("some garbage")

        # Make sure the file is truncated when we run the command again.
        self.runCmd("memory read -f d -c 1 -o '{}' `&argc`".format(memory_read_file))
        check_file_content([golden_output])

        # Make sure the file is appended when we run the command with --append-outfile.
        self.runCmd(
            "memory read -f d -c 1 -o '{}' --append-outfile `&argc`".format(
                memory_read_file
            )
        )
        check_file_content([golden_output, golden_output])



# RUN: %python %s | FileCheck %s

from mlir_standalone.ir import *
from mlir_standalone.dialects import builtin as builtin_d, standalone as standalone_d

with Context():
    standalone_d.register_dialect()
    module = Module.parse(
        """
    %0 = arith.constant 2 : i32
    %1 = standalone.foo %0 : i32
    """
    )
    # CHECK: %[[C:.*]] = arith.constant 2 : i32
    # CHECK: standalone.foo %[[C]] : i32
    print(str(module))



#!/usr/bin/env python

import json
import os
import re
import subprocess
import sys
import argparse


class CrashLogPatcher:
    SYMBOL_REGEX = re.compile(r"^([0-9a-fA-F]+) T _(.*)$")
    UUID_REGEX = re.compile(r"UUID: ([-0-9a-fA-F]+) \(([^\(]+)\) .*")

    def __init__(self, data, binary, offsets, json):
        self.data = data
        self.binary = binary
        self.offsets = offsets
        self.json = json

    def patch_executable(self):
        self.data = self.data.replace("@EXEC@", self.binary)
        self.data = self.data.replace("@NAME@", os.path.basename(self.binary))

    def patch_uuid(self):
        output = subprocess.check_output(["dwarfdump", "--uuid", self.binary]).decode(
            "utf-8"
        )
        m = self.UUID_REGEX.match(output)
        if m:
            self.data = self.data.replace("@UUID@", m.group(1))

    def patch_addresses(self):
        if not self.offsets:
            return
        output = subprocess.check_output(["nm", self.binary]).decode("utf-8")
        for line in output.splitlines():
            m = self.SYMBOL_REGEX.match(line)
            if m:
                address = m.group(1)
                symbol = m.group(2)
                if symbol in self.offsets:
                    patch_addr = int(m.group(1), 16) + int(self.offsets[symbol])
                    if self.json:
                        patch_addr = patch_addr - 0x100000000
                        representation = int
                    else:
                        representation = hex
                    self.data = self.data.replace(
                        "@{}@".format(symbol), str(representation(patch_addr))
                    )

    def remove_metadata(self):
        self.data = self.data[self.data.index("\n") + 1 :]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Crashlog Patcher")
    parser.add_argument("--binary", required=True)
    parser.add_argument("--crashlog", required=True)
    parser.add_argument("--offsets", required=True)
    parser.add_argument("--json", default=False, action="store_true")
    parser.add_argument("--no-metadata", default=False, action="store_true")
    args = parser.parse_args()

    offsets = json.loads(args.offsets)

    with open(args.crashlog, "r") as file:
        data = file.read()

    p = CrashLogPatcher(data, args.binary, offsets, args.json)
    p.patch_executable()
    p.patch_uuid()
    p.patch_addresses()

    if args.no_metadata:
        p.remove_metadata()

    with open(args.crashlog, "w") as file:
        file.write(p.data)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestCase(TestBase):
    @swiftTest
    def test(self):
        """Test that a breakpoint on a property accessor can be set by name."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        for name in (
            "read_only.get",
            "read_write.get",
            "read_write.set",
            "observed.willset",
            "observed.didset",
        ):
            bp = target.BreakpointCreateByName(name, "a.out")
            self.assertEqual(bp.num_locations, 1, f"{name} breakpoint failed")

        # Setting a breakpoint on the name "get" should not create a breakpoint
        # matching property getters. The other accerssor suffixes should also
        # not succeed as bare names.
        for name in ("get", "set", "willset", "didset"):
            bp = target.BreakpointCreateByName(name, "a.out")
            self.assertEqual(bp.num_locations, 0, f"{name} breakpoint unexpected")



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ScopedEnumType(TestBase):
    @skipIf(dwarf_version=["<", "4"])
    def test(self):
        self.build()

        self.main_source = "main.cpp"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", self.main_source_spec
        )
        frame = thread.GetFrameAtIndex(0)

        self.expect("expr f == Foo::FooBar", substrs=["(bool) $0 = true"])

        self.expect_expr("f == Foo::FooBar", result_value="true")
        self.expect_expr("b == BarBar", result_value="true")

        ## b is not a Foo
        value = frame.EvaluateExpression("b == Foo::FooBar")
        self.assertTrue(value.IsValid())
        self.assertFalse(value.GetError().Success())

        ## integral is not implicitly convertible to a scoped enum
        value = frame.EvaluateExpression("1 == Foo::FooBar")
        self.assertTrue(value.IsValid())
        self.assertFalse(value.GetError().Success())



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftStaticFramework(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)
    NO_DEBUG_INFO_TESTCASE = True

    @swiftTest
    @skipIf(oslist=no_match(["macosx"]))
    def test_static_framework(self):
        """Make sure LLDB doesn't attempt to import static frameworks"""
        n = 10
        for i in range(n):
            with open(self.getBuildArtifact("a%d.swift"%i), "w") as f:
                f.write("public struct A%d { public init() {} }\n"%i)
        src = self.getBuildArtifact("main.swift")
        with open(src, "w") as f:
            f.write("func use<T>(_ t: T) {}\n")
            for i in range(n):
                f.write("import a%d\n"%i)
            for i in range(n):
                f.write("let v%d = A%d()\n"%(i,i))
                f.write("use(v%d)\n"%(i))
            f.write('print("break here")\n')
        self.build(dictionary={"N": "%d"%(n-1)})
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec(src))

        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)
        # Run the expression evaluator to trigger import of all static
        # frameworks.  The expression imports a new dynamic framework
        # as a sanity check to make sure this still works.
        self.expect("expression -- import Dylib")
        import_a0 = 0
        load_a0 = 0
        load_dylib = 0
        import io
        with open(log, "r", encoding='utf-8') as logfile:
            for line in logfile:
                if 'Loading linked framework "Dylib"' in line:
                    load_dylib += 1
                elif "Importing module a0" in line:
                    import_a0 += 1
                elif 'Loading linked framework "a0"' in line:
                    load_a0 += 1

        self.assertGreater(import_a0, 0, "sanity check failed")
        self.assertEqual(load_a0, 0, "attempted to import a0 again")
        self.assertEqual(load_dylib, 1, "failed to import dylib")



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil

class TestSwiftExprMissingType(lldbtest.TestBase):
    @swiftTest
    def test(self):
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        self.expect("frame variable invisible", substrs=["1", "2"])
        self.expect("expr invisible", error=True,
                    substrs=["Missing type debug info", "invisible"])



# TestTsanSwift.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test Swift support of TSan.
"""
import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2
import json


class TsanSwiftTestCase(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    @skipIfLinux
    @skipUnlessSwiftThreadSanitizer
    @skipIfAsan # This test does not behave reliable with an ASANified LLDB.
    def test_tsan_swift(self):
        self.build()
        self.do_test()

    def setUp(self):
        lldbtest.TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def do_test(self):
        exe_name = "a.out"
        exe = self.getBuildArtifact(exe_name)

        # Create the target
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, lldbtest.VALID_TARGET)

        runtimes = []
        for m in target.module_iter():
            libspec = m.GetFileSpec()
            if "clang_rt" in libspec.GetFilename():
                runtimes.append(os.path.join(libspec.GetDirectory(), libspec.GetFilename()))
        self.registerSharedLibrariesWithTarget(target, runtimes)

        self.runCmd("run")
        self.runCmd("bt")

        stop_reason = self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason()
        if stop_reason == lldb.eStopReasonExec:
            # On OS X 10.10 and older, we need to re-exec to enable
            # interceptors.
            self.runCmd("continue")

        # the stop reason of the thread should be a TSan report.
        self.expect("thread list", "A data race should be detected",
                    substrs=['stopped', 'stop reason = Data race detected'])

        self.assertEqual(
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason(),
            lldb.eStopReasonInstrumentation)

        self.expect(
            "thread info -s",
            "The extended stop info should contain the TSan provided fields",
            substrs=[
                "instrumentation_class",
                "description",
                "mops"])

        output_lines = self.res.GetOutput().split('\n')
        json_line = '\n'.join(output_lines[2:])
        data = json.loads(json_line)
        self.assertEqual(data["instrumentation_class"], "ThreadSanitizer")
        self.assertEqual(data["issue_type"], "data-race")
        self.assertEqual(len(data["mops"]), 2)
        self.assertTrue(data["location_filename"].endswith("/main.swift"))
        self.assertEqual(
            data["location_line"],
            lldbtest.line_number(
                'main.swift',
                '// global variable'))



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

"""Experimental MLIR-PyTACO with sparse tensor support.

See http://tensor-compiler.org/ for TACO tensor compiler.

This module implements the PyTACO API for writing a tensor to a file or reading
a tensor from a file.

See the following links for Matrix Market Exchange (.mtx) format and FROSTT
(.tns) format:
  https://math.nist.gov/MatrixMarket/formats.html
  http://frostt.io/tensors/file-formats.html
"""

from typing import List, TextIO

from . import mlir_pytaco

# Define the type aliases so that we can write the implementation here as if
# it were part of mlir_pytaco.py.
Tensor = mlir_pytaco.Tensor
Format = mlir_pytaco.Format
DType = mlir_pytaco.DType
Type = mlir_pytaco.Type

# Constants used in the implementation.
_MTX_FILENAME_SUFFIX = ".mtx"
_TNS_FILENAME_SUFFIX = ".tns"


def read(filename: str, fmt: Format, dtype: DType = DType(Type.FLOAT32)) -> Tensor:
    """Inputs a tensor from a given file.

    The name suffix of the file specifies the format of the input tensor. We
    currently only support .mtx format for support sparse tensors.

    Args:
      filename: A string input filename.
      fmt: The storage format of the tensor.
      dtype: The data type, default to float32.

    Raises:
      ValueError: If filename doesn't end with .mtx or .tns, or fmt is not an
      instance of Format or fmt is not a sparse tensor.
    """
    if not isinstance(filename, str) or (
        not filename.endswith(_MTX_FILENAME_SUFFIX)
        and not filename.endswith(_TNS_FILENAME_SUFFIX)
    ):
        raise ValueError(
            "Expected string filename ends with "
            f"{_MTX_FILENAME_SUFFIX} or {_TNS_FILENAME_SUFFIX}: "
            f"{filename}."
        )

    return Tensor.from_file(filename, fmt, dtype)


def write(filename: str, tensor: Tensor) -> None:
    """Outputs a tensor to a given file.

    The name suffix of the file specifies the format of the output. We currently
    only support .tns format.

    Args:
      filename: A string output filename.
      tensor: The tensor to output.

    Raises:
      ValueError: If filename doesn't end with .tns or tensor is not a Tensor.
    """
    if not isinstance(filename, str) or not filename.endswith(_TNS_FILENAME_SUFFIX):
        raise ValueError(
            "Expected string filename ends with" f" {_TNS_FILENAME_SUFFIX}: {filename}."
        )
    if not isinstance(tensor, Tensor):
        raise ValueError(f"Expected a Tensor object: {tensor}.")

    tensor.to_file(filename)



"""
Test AArch64 dynamic register sets
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class RegisterCommandsTestCase(TestBase):
    def check_sve_register_size(self, set, name, expected):
        reg_value = set.GetChildMemberWithName(name)
        self.assertTrue(reg_value.IsValid(), "Expected a register named %s" % (name))
        self.assertEqual(
            reg_value.GetByteSize(),
            expected,
            "Expected a register %s size == %i bytes" % (name, expected),
        )

    def sve_regs_read_dynamic(self, sve_registers):
        vg_reg = sve_registers.GetChildMemberWithName("vg")
        vg_reg_value = sve_registers.GetChildMemberWithName("vg").GetValueAsUnsigned()

        z_reg_size = vg_reg_value * 8
        p_reg_size = int(z_reg_size / 8)

        for i in range(32):
            z_regs_value = (
                "{"
                + " ".join("0x{:02x}".format(i + 1) for _ in range(z_reg_size))
                + "}"
            )
            self.expect("register read z%i" % (i), substrs=[z_regs_value])

        # Set P registers with random test values. The P registers are predicate
        # registers, which hold one bit for each byte available in a Z register.
        # For below mentioned values of P registers, P(0,5,10,15) will have all
        # Z register lanes set while P(4,9,14) will have no lanes set.
        p_value_bytes = ["0xff", "0x55", "0x11", "0x01", "0x00"]
        for i in range(16):
            p_regs_value = (
                "{" + " ".join(p_value_bytes[i % 5] for _ in range(p_reg_size)) + "}"
            )
            self.expect("register read p%i" % (i), substrs=[p_regs_value])

        self.expect("register read ffr", substrs=[p_regs_value])

        for i in range(32):
            z_regs_value = (
                "{"
                + " ".join("0x{:02x}".format(32 - i) for _ in range(z_reg_size))
                + "}"
            )
            self.runCmd("register write z%i '%s'" % (i, z_regs_value))
            self.expect("register read z%i" % (i), substrs=[z_regs_value])

        for i in range(16):
            p_regs_value = (
                "{"
                + " ".join("0x{:02x}".format(16 - i) for _ in range(p_reg_size))
                + "}"
            )
            self.runCmd("register write p%i '%s'" % (i, p_regs_value))
            self.expect("register read p%i" % (i), substrs=[p_regs_value])

        p_regs_value = (
            "{" + " ".join("0x{:02x}".format(8) for _ in range(p_reg_size)) + "}"
        )
        self.runCmd("register write ffr " + "'" + p_regs_value + "'")
        self.expect("register read ffr", substrs=[p_regs_value])

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_aarch64_dynamic_regset_config(self):
        """Test AArch64 Dynamic Register sets configuration."""
        self.build()
        self.line = line_number("main.c", "// Set a break point here.")

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()
        thread = process.GetThreadAtIndex(0)
        currentFrame = thread.GetFrameAtIndex(0)

        for registerSet in currentFrame.GetRegisters():
            if "Scalable Vector Extension Registers" in registerSet.GetName():
                self.assertTrue(
                    self.isAArch64SVE(),
                    "LLDB enabled AArch64 SVE register set when it was disabled by target.",
                )
                self.sve_regs_read_dynamic(registerSet)
            if "MTE Control Register" in registerSet.GetName():
                self.assertTrue(
                    self.isAArch64MTE(),
                    "LLDB enabled AArch64 MTE register set when it was disabled by target.",
                )
                self.runCmd("register write mte_ctrl 0x7fff9")
                self.expect(
                    "register read mte_ctrl", substrs=["mte_ctrl = 0x000000000007fff9"]
                )
            if "Pointer Authentication Registers" in registerSet.GetName():
                self.assertTrue(
                    self.isAArch64PAuth(),
                    "LLDB enabled AArch64 Pointer Authentication register set when it was disabled by target.",
                )
                self.expect("register read data_mask", substrs=["data_mask = 0x"])
                self.expect("register read code_mask", substrs=["code_mask = 0x"])



"""
Test whether a process started by lldb has no extra file descriptors open.
"""


import lldb
from lldbsuite.test import lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class AvoidsFdLeakTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # The check for descriptor leakage needs to be implemented differently
    # here.
    @skipIfWindows
    @skipIfTargetAndroid()  # Android have some other file descriptors open by the shell
    @skipIfDarwinEmbedded  # <rdar://problem/33888742>  # debugserver on ios has an extra fd open on launch
    def test_fd_leak_basic(self):
        self.do_test([])

    # The check for descriptor leakage needs to be implemented differently
    # here.
    @skipIfWindows
    @skipIfTargetAndroid()  # Android have some other file descriptors open by the shell
    @skipIfDarwinEmbedded  # <rdar://problem/33888742>  # debugserver on ios has an extra fd open on launch
    def test_fd_leak_log(self):
        self.do_test(["log enable -f '/dev/null' lldb commands"])

    def do_test(self, commands):
        self.build()
        exe = self.getBuildArtifact("a.out")

        for c in commands:
            self.runCmd(c)

        target = self.dbg.CreateTarget(exe)

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        self.assertEqual(
            process.GetState(), lldb.eStateExited, "Process should have exited."
        )
        self.assertEqual(
            process.GetExitStatus(),
            0,
            "Process returned non-zero status. Were incorrect file descriptors passed?",
        )

    # The check for descriptor leakage needs to be implemented differently
    # here.
    @skipIfWindows
    @skipIfTargetAndroid()  # Android have some other file descriptors open by the shell
    @skipIfDarwinEmbedded  # <rdar://problem/33888742>  # debugserver on ios has an extra fd open on launch
    def test_fd_leak_multitarget(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        breakpoint = target.BreakpointCreateBySourceRegex(
            "Set breakpoint here", lldb.SBFileSpec("main.c", False)
        )
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        process1 = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process1, PROCESS_IS_VALID)
        self.assertEqual(
            process1.GetState(), lldb.eStateStopped, "Process should have been stopped."
        )

        target2 = self.dbg.CreateTarget(exe)
        process2 = target2.LaunchSimple(
            None, None, self.get_process_working_directory()
        )
        self.assertTrue(process2, PROCESS_IS_VALID)

        self.assertEqual(
            process2.GetState(), lldb.eStateExited, "Process should have exited."
        )
        self.assertEqual(
            process2.GetExitStatus(),
            0,
            "Process returned non-zero status. Were incorrect file descriptors passed?",
        )



from lldbsuite.test.lldbtest import Base
from lldbsuite.test.decorators import *


class TestDecorators(Base):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(debug_info="dwarf")
    def test_decorator_skip_no_debug_info(self):
        """Test that specifying a debug info category works for a NO_DEBUG_INFO_TESTCASE"""
        pass




"""
Test that printing C++ types with typedefs, using, and Swift typealiases print correctly in Swift
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftForwardInteropTypedefType(TestBase):

    @swiftTest
    def test_class(self):
        self.build()
        
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        self.expect('v typedef', substrs=['TypedefedCxxClass', 'a1', '10', 'a2', '20', 'a3', '30'])
        self.expect('expr typedef', substrs=['TypedefedCxxClass', 'a1', '10', 'a2', '20', 'a3', '30'])

        self.expect('v using', substrs=['UsingCxxClass', 'a1', '10', 'a2', '20', 'a3', '30'])
        self.expect('expr using', substrs=['UsingCxxClass', 'a1', '10', 'a2', '20', 'a3', '30'])

        self.expect('v typealiased', substrs=['TypeAliased', 'a1', '10', 'a2', '20', 'a3', '30'])
        self.expect('expr typealiased', substrs=['TypeAliased', 'a1', '10', 'a2', '20', 'a3', '30'])



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# CHECK-LABEL: TEST: testAffineMapCapsule
@run
def testAffineMapCapsule():
    with Context() as ctx:
        am1 = AffineMap.get_empty(ctx)
    # CHECK: mlir.ir.AffineMap._CAPIPtr
    affine_map_capsule = am1._CAPIPtr
    print(affine_map_capsule)
    am2 = AffineMap._CAPICreate(affine_map_capsule)
    assert am2 == am1
    assert am2.context is ctx


# CHECK-LABEL: TEST: testAffineMapGet
@run
def testAffineMapGet():
    with Context() as ctx:
        d0 = AffineDimExpr.get(0)
        d1 = AffineDimExpr.get(1)
        c2 = AffineConstantExpr.get(2)

        # CHECK: (d0, d1)[s0, s1, s2] -> ()
        map0 = AffineMap.get(2, 3, [])
        print(map0)

        # CHECK: (d0, d1)[s0, s1, s2] -> (d1, 2)
        map1 = AffineMap.get(2, 3, [d1, c2])
        print(map1)

        # CHECK: () -> (2)
        map2 = AffineMap.get(0, 0, [c2])
        print(map2)

        # CHECK: (d0, d1) -> (d0, d1)
        map3 = AffineMap.get(2, 0, [d0, d1])
        print(map3)

        # CHECK: (d0, d1) -> (d1)
        map4 = AffineMap.get(2, 0, [d1])
        print(map4)

        # CHECK: (d0, d1, d2) -> (d2, d0, d1)
        map5 = AffineMap.get_permutation([2, 0, 1])
        print(map5)

        assert map1 == AffineMap.get(2, 3, [d1, c2])
        assert AffineMap.get(0, 0, []) == AffineMap.get_empty()
        assert map2 == AffineMap.get_constant(2)
        assert map3 == AffineMap.get_identity(2)
        assert map4 == AffineMap.get_minor_identity(2, 1)

        try:
            AffineMap.get(1, 1, [1])
        except RuntimeError as e:
            # CHECK: Invalid expression when attempting to create an AffineMap
            print(e)

        try:
            AffineMap.get(1, 1, [None])
        except RuntimeError as e:
            # CHECK: Invalid expression (None?) when attempting to create an AffineMap
            print(e)

        try:
            AffineMap.get_permutation([1, 0, 1])
        except RuntimeError as e:
            # CHECK: Invalid permutation when attempting to create an AffineMap
            print(e)

        try:
            map3.get_submap([42])
        except ValueError as e:
            # CHECK: result position out of bounds
            print(e)

        try:
            map3.get_minor_submap(42)
        except ValueError as e:
            # CHECK: number of results out of bounds
            print(e)

        try:
            map3.get_major_submap(42)
        except ValueError as e:
            # CHECK: number of results out of bounds
            print(e)


# CHECK-LABEL: TEST: testAffineMapDerive
@run
def testAffineMapDerive():
    with Context() as ctx:
        map5 = AffineMap.get_identity(5)

        # CHECK: (d0, d1, d2, d3, d4) -> (d1, d2, d3)
        map123 = map5.get_submap([1, 2, 3])
        print(map123)

        # CHECK: (d0, d1, d2, d3, d4) -> (d0, d1)
        map01 = map5.get_major_submap(2)
        print(map01)

        # CHECK: (d0, d1, d2, d3, d4) -> (d3, d4)
        map34 = map5.get_minor_submap(2)
        print(map34)


# CHECK-LABEL: TEST: testAffineMapProperties
@run
def testAffineMapProperties():
    with Context():
        d0 = AffineDimExpr.get(0)
        d1 = AffineDimExpr.get(1)
        d2 = AffineDimExpr.get(2)
        map1 = AffineMap.get(3, 0, [d2, d0])
        map2 = AffineMap.get(3, 0, [d2, d0, d1])
        map3 = AffineMap.get(3, 1, [d2, d0, d1])
        # CHECK: False
        print(map1.is_permutation)
        # CHECK: True
        print(map1.is_projected_permutation)
        # CHECK: True
        print(map2.is_permutation)
        # CHECK: True
        print(map2.is_projected_permutation)
        # CHECK: False
        print(map3.is_permutation)
        # CHECK: False
        print(map3.is_projected_permutation)


# CHECK-LABEL: TEST: testAffineMapExprs
@run
def testAffineMapExprs():
    with Context():
        d0 = AffineDimExpr.get(0)
        d1 = AffineDimExpr.get(1)
        d2 = AffineDimExpr.get(2)
        map3 = AffineMap.get(3, 1, [d2, d0, d1])

        # CHECK: 3
        print(map3.n_dims)
        # CHECK: 4
        print(map3.n_inputs)
        # CHECK: 1
        print(map3.n_symbols)
        assert map3.n_inputs == map3.n_dims + map3.n_symbols

        # CHECK: 3
        print(len(map3.results))
        for expr in map3.results:
            # CHECK: d2
            # CHECK: d0
            # CHECK: d1
            print(expr)
        for expr in map3.results[-1:-4:-1]:
            # CHECK: d1
            # CHECK: d0
            # CHECK: d2
            print(expr)
        assert list(map3.results) == [d2, d0, d1]


# CHECK-LABEL: TEST: testCompressUnusedSymbols
@run
def testCompressUnusedSymbols():
    with Context() as ctx:
        d0, d1, d2 = (AffineDimExpr.get(0), AffineDimExpr.get(1), AffineDimExpr.get(2))
        s0, s1, s2 = (
            AffineSymbolExpr.get(0),
            AffineSymbolExpr.get(1),
            AffineSymbolExpr.get(2),
        )
        maps = [
            AffineMap.get(3, 3, [d2, d0, d1]),
            AffineMap.get(3, 3, [d2, d0 + s2, d1]),
            AffineMap.get(3, 3, [d1, d2, d0]),
        ]

        compressed_maps = AffineMap.compress_unused_symbols(maps, ctx)

        #      CHECK: AffineMap((d0, d1, d2)[s0, s1, s2] -> (d2, d0, d1))
        # CHECK-SAME: AffineMap((d0, d1, d2)[s0, s1, s2] -> (d2, d0 + s2, d1))
        # CHECK-SAME: AffineMap((d0, d1, d2)[s0, s1, s2] -> (d1, d2, d0))
        print(maps)

        #      CHECK: AffineMap((d0, d1, d2)[s0] -> (d2, d0, d1))
        # CHECK-SAME: AffineMap((d0, d1, d2)[s0] -> (d2, d0 + s0, d1))
        # CHECK-SAME: AffineMap((d0, d1, d2)[s0] -> (d1, d2, d0))
        print(compressed_maps)


# CHECK-LABEL: TEST: testReplace
@run
def testReplace():
    with Context() as ctx:
        d0, d1, d2 = (AffineDimExpr.get(0), AffineDimExpr.get(1), AffineDimExpr.get(2))
        s0, s1, s2 = (
            AffineSymbolExpr.get(0),
            AffineSymbolExpr.get(1),
            AffineSymbolExpr.get(2),
        )
        map1 = AffineMap.get(3, 3, [d2, d0 + s1 + s2, d1 + s0])

        replace0 = map1.replace(s0, AffineConstantExpr.get(42), 3, 3)
        replace1 = map1.replace(s1, AffineConstantExpr.get(42), 3, 3)
        replace3 = map1.replace(s2, AffineConstantExpr.get(42), 3, 2)

        # CHECK: (d0, d1, d2)[s0, s1, s2] -> (d2, d0 + s1 + s2, d1 + 42)
        print(replace0)

        # CHECK: (d0, d1, d2)[s0, s1, s2] -> (d2, d0 + s2 + 42, d1 + s0)
        print(replace1)

        # CHECK: (d0, d1, d2)[s0, s1] -> (d2, d0 + s1 + 42, d1 + s0)
        print(replace3)


# CHECK-LABEL: TEST: testHash
@run
def testHash():
    with Context():
        d0, d1 = AffineDimExpr.get(0), AffineDimExpr.get(1)
        m1 = AffineMap.get(2, 0, [d0, d1])
        m2 = AffineMap.get(2, 0, [d1, d0])
        assert hash(m1) == hash(AffineMap.get(2, 0, [d0, d1]))

        dictionary = dict()
        dictionary[m1] = 1
        dictionary[m2] = 2
        assert m1 in dictionary



"""
Test SubstTemplateTypeParam types which are produced as type sugar
when template type parameters are used for example as field types.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test import decorators


class TestCase(TestBase):
    def test_typedef(self):
        target = self.dbg.GetDummyTarget()

        # Declare a template class with a field that uses the template type
        # parameter.
        opts = lldb.SBExpressionOptions()
        opts.SetTopLevel(True)
        result = target.EvaluateExpression(
            "template <typename T> struct X { T f; };", opts
        )
        # FIXME: This fails with "Couldn't find $__lldb_expr() in the module"
        # but it should succeed. The fact that this code has nothing to run
        # shouldn't be an error.
        # self.assertSuccess(result.GetError())

        # Instantiate and produce a value with that template as the type.
        # The field in the value will have a SubstTemplateTypeParam that
        # should behave like a normal field.
        result = target.EvaluateExpression("X<int> x; x.f = 123; x")
        self.assertEqual(result.GetNumChildren(), 1)
        self.assertEqual(result.GetChildAtIndex(0).GetTypeName(), "int")
        self.assertEqual(result.GetChildAtIndex(0).GetValue(), "123")



# TestSwiftDWARFImporterBridgingHeader.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2019 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftDWARFImporterBridgingHeader(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    def build(self):
        include = self.getBuildArtifact('include')
        inputs = self.getSourcePath('Inputs')
        lldbutil.mkdir_p(include)
        import shutil
        for f in ['c-header.h']:
            shutil.copyfile(os.path.join(inputs, f), os.path.join(include, f))

        super(TestSwiftDWARFImporterBridgingHeader, self).build()

        # Remove the header files to prevent ClangImporter loading the
        # bridging header from source.
        self.assertTrue(os.path.isdir(include))
        shutil.rmtree(include)

    @skipIf(archs=['ppc64le'], bugnumber='SR-10214')
    # This test needs a working Remote Mirrors implementation.
    @skipIf(oslist=['windows'])
    # FIMXE: This broke when adapting the swift-driver based Makefile.rules
    @expectedFailureAll(oslist=['linux'])
    # We delete the pch that would contains the debug info as part of the setup.
    #@skipIf(debug_info=no_match(["dsym"]))
    @swiftTest
    def test_dwarf_importer(self):
        lldb.SBDebugger.MemoryPressureDetected()
        self.runCmd("settings set symbols.use-swift-dwarfimporter true")
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))
        lldbutil.check_variable(self,
                                target.FindFirstGlobalVariable("pureSwift"),
                                value="42")
        lldbutil.check_variable(self,
                                target.FindFirstGlobalVariable("point"),
                                typename='bridging-header.h.Point', num_children=2)
        self.expect("ta v -d no-dyn point", substrs=["x = 1", "y = 2"])
        self.expect("ta v -d no-dyn swiftStructCMember",
                    substrs=[
                        # FIXME: This doesn't even work with the original bridging header!
                        #"point", "x = 3", "y = 4",
                        "swift struct c member"])
        process.Clear()
        target.Clear()
        lldb.SBDebugger.MemoryPressureDetected()



"""
Test that we correctly handle inline namespaces.
"""

import lldb

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestInlineNamespace(TestBase):
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        # The 'A::B::f' function must be found via 'A::f' as 'B' is an inline
        # namespace.
        self.expect_expr("A::f()", result_type="int", result_value="3")
        # But we should still find the function when we pretend the inline
        # namespace is not inline.
        self.expect_expr("A::B::f()", result_type="int", result_value="3")

        self.expect_expr("A::B::global_var", result_type="int", result_value="0")
        # FIXME: should be ambiguous lookup but ClangExpressionDeclMap takes
        #        first global variable that the lookup found, which in this case
        #        is A::B::global_var
        self.expect_expr("A::global_var", result_type="int", result_value="0")

        self.expect_expr("A::B::C::global_var", result_type="int", result_value="1")
        self.expect_expr("A::C::global_var", result_type="int", result_value="1")

        self.expect_expr("A::B::D::nested_var", result_type="int", result_value="2")
        self.expect_expr("A::D::nested_var", result_type="int", result_value="2")
        self.expect_expr("A::B::nested_var", result_type="int", result_value="2")
        self.expect_expr("A::nested_var", result_type="int", result_value="2")

        self.expect_expr("A::E::F::other_var", result_type="int", result_value="3")
        self.expect_expr("A::E::other_var", result_type="int", result_value="3")

        self.expect(
            "expr A::E::global_var",
            error=True,
            substrs=["no member named 'global_var' in namespace 'A::E'"],
        )
        self.expect(
            "expr A::E::F::global_var",
            error=True,
            substrs=["no member named 'global_var' in namespace 'A::E::F'"],
        )

        self.expect(
            "expr A::other_var",
            error=True,
            substrs=["no member named 'other_var' in namespace 'A'"],
        )
        self.expect(
            "expr A::B::other_var",
            error=True,
            substrs=["no member named 'other_var' in namespace 'A::B'"],
        )
        self.expect(
            "expr B::other_var",
            error=True,
            substrs=["no member named 'other_var' in namespace 'A::B'"],
        )

        # 'frame variable' can correctly distinguish between A::B::global_var and A::global_var
        gvars = self.target().FindGlobalVariables("A::global_var", 10)
        self.assertEqual(len(gvars), 1)
        self.assertEqual(gvars[0].GetValueAsSigned(), 4)

        self.expect("frame variable A::global_var", substrs=["(int) A::global_var = 4"])
        self.expect(
            "frame variable A::B::global_var", substrs=["(int) A::B::global_var = 0"]
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    def test(self):
        self.build()
        self.dbg.CreateTarget(self.getBuildArtifact("a.out"))

        # The offset of f2 should be 8 because of `alignas(8)`.
        self.expect_expr("(intptr_t)&d3g.f2 - (intptr_t)&d3g", result_value="8")



"""
Test that we obey thread conditioned breakpoints and expression
conditioned breakpoints simultaneously
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ThreadSpecificBreakPlusConditionTestCase(TestBase):
    # test frequently times out or hangs
    @skipIfDarwin
    # hits break in another thread in testrun
    @add_test_categories(["pyapi"])
    @expectedFlakeyNetBSD
    @skipIfWindows  # This test is flaky on Windows
    def test_python(self):
        """Test that we obey thread conditioned breakpoints."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        main_source_spec = lldb.SBFileSpec("main.cpp")

        # Set a breakpoint in the thread body, and make it active for only the
        # first thread.
        break_thread_body = target.BreakpointCreateBySourceRegex(
            "Break here in thread body.", main_source_spec
        )
        self.assertTrue(
            break_thread_body.IsValid() and break_thread_body.GetNumLocations() > 0,
            "Failed to set thread body breakpoint.",
        )

        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_thread_body)

        victim_thread = threads[0]

        # Pick one of the threads, and change the breakpoint so it ONLY stops for this thread,
        # but add a condition that it won't stop for this thread's my_value.  The other threads
        # pass the condition, so they should stop, but if the thread-specification is working
        # they should not stop.  So nobody should hit the breakpoint anymore, and we should
        # just exit cleanly.

        frame = victim_thread.GetFrameAtIndex(0)
        value = frame.FindVariable("my_value").GetValueAsSigned(0)
        self.assertTrue(
            value > 0 and value < 11, "Got a reasonable value for my_value."
        )

        cond_string = "my_value != %d" % (value)

        break_thread_body.SetThreadID(victim_thread.GetThreadID())
        break_thread_body.SetCondition(cond_string)

        process.Continue()

        next_stop_state = process.GetState()
        self.assertEqual(
            next_stop_state,
            lldb.eStateExited,
            "We should have not hit the breakpoint again.",
        )



"""
Test CFUUID object description.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil


class TestCase(TestBase):
    @swiftTest
    @skipUnlessFoundation
    def test(self):
        """Test CFUUID object description prints the UUID string."""
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.swift"))
        # Swift type validation fails in IsPossibleDynamicType rdar://109611675
        self.runCmd("settings set symbols.swift-validate-typesystem false")
        uuid = "68753A44-4D6F-1226-9C60-0050E4C00067"
        self.expect("frame variable -O uuid", substrs=[uuid])
        self.expect("dwim-print -O -- uuid", substrs=[uuid])
        self.expect("expression -O -- uuid", substrs=[uuid])



"""
Test the 'gui' shortcut 'b' (toggle breakpoint).
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestGuiBasicDebugCommandTest(PExpectTest):
    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIfCursesSupportMissing
    @skipIf(oslist=["linux"], archs=["arm", "aarch64"])
    def test_gui(self):
        self.build()

        self.launch(executable=self.getBuildArtifact("a.out"), dimensions=(100, 500))
        self.expect(
            'br set -o true -f main.c -p "// First break here"',
            substrs=["Breakpoint 1", "address ="],
        )
        self.expect("run", substrs=["stop reason ="])

        self.child.sendline("breakpoint list")
        self.child.expect_exact("No breakpoints currently set.")

        escape_key = chr(27).encode()
        down_key = chr(27) + "OB"  # for vt100 terminal (lldbexpect sets TERM=vt100)

        # Start the GUI.
        self.child.sendline("gui")
        self.child.expect_exact("Sources")  # wait for gui

        # Go to next line, set a breakpoint.
        self.child.send(down_key)
        self.child.send("b")
        self.child.send(escape_key)
        self.expect_prompt()
        self.child.sendline("breakpoint list")
        self.child.expect("2: file = '[^']*main.c', line = 3,.*")
        self.child.sendline("gui")
        self.child.expect_exact("Sources")

        # Go two lines down ("gui" resets position), set a breakpoint.
        self.child.send(down_key)
        self.child.send(down_key)
        self.child.send("b")
        self.child.send(escape_key)
        self.expect_prompt()
        self.child.sendline("breakpoint list")
        self.child.expect("2: file = '[^']*main.c', line = 3,")
        self.child.expect("3: file = '[^']*main.c', line = 4,")
        self.child.sendline("gui")
        self.child.expect_exact("Sources")

        # Toggle both the breakpoints (remove them).
        self.child.send(down_key)
        self.child.send("b")
        self.child.send(down_key)
        self.child.send("b")
        self.child.send(escape_key)
        self.expect_prompt()
        self.child.sendline("breakpoint list")
        self.child.expect_exact("No breakpoints currently set.")
        self.child.sendline("gui")
        self.child.expect_exact("Sources")

        # Press escape to quit the gui
        self.child.send(escape_key)

        self.expect_prompt()
        self.quit()



# TestSwiftValueOfOptionals.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Check that trying to read an optional's numeric value doesn't crash LLDB
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftValueOfOptionalType(TestBase):
    @swiftTest
    def test_swift_value_optional_type(self):
        """Check that trying to read an optional's numeric value doesn't crash LLDB"""
        self.build()
        self.do_check()

    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def do_check(self):
        """Check that trying to read an optional's numeric value doesn't crash LLDB"""
        s = self.frame().FindVariable("s")
        self.assertTrue(s.GetValueAsSigned(0) == 0, "reading value fails")
        self.assertTrue(s.GetValueAsSigned(1) == 1, "reading value fails")
        self.assertTrue(s.GetValueAsUnsigned(0) == 0, "reading value fails")
        self.assertTrue(s.GetValueAsUnsigned(1) == 1, "reading value fails")




"""
Base class for hardware breakpoints tests.
"""

from lldbsuite.test.lldbtest import *


class HardwareBreakpointTestBase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def supports_hw_breakpoints(self):
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)
        self.runCmd("breakpoint set -b main --hardware")
        self.runCmd("run")
        if "stopped" in self.res.GetOutput():
            return "Hardware breakpoints are supported"
        return None



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftDifferentABIName(TestBase):
    @swiftTest
    def test(self):
        self.build()

        _, _, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )

        self.expect("frame variable s", 
                    substrs=["Struct", "s = ", "field = 42"])
        self.expect("expr s", substrs=["Struct", "field = 42"])
        self.expect("expr -O -- s", substrs=["Struct", "- field : 42"])



import lldb
import side_effect


class Resolver:
    """This resolver class is just so I can read out the extra_args."""

    def __init__(self, bkpt, extra_args, dict):
        self.bkpt = bkpt
        side_effect.g_extra_args = extra_args

    def __callback__(self, sym_ctx):
        """Doesn't actually do anything."""
        return

    def get_short_help(self):
        return "I am a python breakpoint resolver that does nothing"
